/////////////////////////////////////////////////////////////////////////////
// Name:        V1724_board.cpp
// Purpose:     
// Author:      NDA
// Modified by: 
// Created:     04/03/06 14:18:40
// RCS-ID:      
// Copyright:   CAEN S.p.A. All rights reserved
// Licence:     
/////////////////////////////////////////////////////////////////////////////


#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "V1724_board.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "v1724_board.h"
#include "v1724_board_channel.h"
#include "appsettings.h"

V1724Board::V1724Board( int board_index, void (*scope_refresh)( bool), wxMutex *p_data_mutex, AppSettings *p_app_settings): 
						GenericBoard( board_index, scope_refresh, p_data_mutex, p_app_settings) 
{
	this->m_p_data= (cvt_board_data*)new cvt_V1724_data;
}

V1724Board::~V1724Board(void)
{
	if( this->m_p_data!= NULL)
	{
		cvt_V1724_close( (cvt_V1724_data* )this->m_p_data);
		delete (cvt_V1724_data* )this->m_p_data;
		this->m_p_data= NULL;
	}
}

bool V1724Board::Open( long vme_handle)
{
	return cvt_V1724_open( (cvt_V1724_data* )this->m_p_data, this->m_p_data->m_base_address, vme_handle)!= FALSE;
}

bool V1724Board::Init( void)
{
	// Memory chip enable
	UINT32 set_msk= CVT_V1724_CHCTRL_MEM_ENABLE_MSK| CVT_V1724_CHCTRL_MEM_RANDOM_ACC_MSK;
	// DbgOnly: set to debug mode to autogenerate samples
	//set_msk|= CVT_V1724_CHCTRL_MEM_TEST_MSK;
	if( !cvt_write_reg( this->m_p_data, CVT_V1724_BROAD_CH_SET_CTRL_INDEX, &set_msk))
		return false;

	// 
	// Font Panel IO
	if( !this->SetFrontPanelIO( this->m_p_app_settings->m_use_TTL))
		return false;

	return true;
}

GenericBoardChannel* V1724Board::CreateChannel( int ch_index, void (* scope_refresh)( bool), wxMutex *p_data_mutex)
{
	return new V1724BoardChannel( ch_index, this->m_p_data, scope_refresh, p_data_mutex, this->m_p_app_settings);
}

bool V1724Board::WriteSampleBufferSize( void)
{
	return cvt_V1724_set_acquisition_mode( (cvt_V1724_data* )this->m_p_data, false, ( CVT_V1724_CH_BLKSIZE) ( CVT_V1724_CHBKSZ_512- this->m_p_app_settings->m_sample_block_size))!= FALSE;
}

bool V1724Board::WriteTriggerMode( bool started)
{
	bool falling_edge_enable= this->m_p_app_settings->m_trigger_edge_type== AppSettings::FALLING_EDGE;
	bool trigger_out_enable= this->m_p_app_settings->m_trigger_edge_type!= AppSettings::NO_TRIGGER;
	unsigned char ch_msk= 0;
	int i;

	// HACK: delete when fixed on next firmware release
	if( this->m_p_app_settings->m_post_trigger< 5)
		this->m_p_app_settings->m_post_trigger= 5;

	for( i= 0; i< (int)this->m_channel_array.GetCount(); i++)
	{
		ch_msk|= ( (GenericBoardChannel*)this->m_channel_array[i])->m_enabled? ( 1<< i): 0;
	}
	if( !( started? 
		cvt_V1724_set_trigger_mode( (cvt_V1724_data* )this->m_p_data, falling_edge_enable, true, trigger_out_enable, this->m_p_app_settings->m_trigger_msk& ( int)AppSettings::EXT_TRIGGER_MSK, true, ch_msk, false, this->m_p_app_settings->m_post_trigger):
		cvt_V1724_set_trigger_mode( (cvt_V1724_data* )this->m_p_data, false, false, false, false, false, 0x00, false, this->m_p_app_settings->m_post_trigger)))
			return false;
	// Data clear
	return cvt_V1724_data_clear( (cvt_V1724_data* )this->m_p_data)!= FALSE;
}

bool V1724Board::WriteSoftwareTrigger( void)
{
	wxMutexLocker lock( this->m_p_app_settings->m_mutex);

	return cvt_V1724_software_trigger( (cvt_V1724_data* )this->m_p_data)!= FALSE;
}

bool V1724Board::SetFrontPanelIO( bool use_TTL)
{
	return cvt_V1724_set_front_panel_IO( (cvt_V1724_data* )this->m_p_data, use_TTL)!= FALSE;
}


