// Name:        appsettings.cpp
// Purpose:     
// Author:      NDA
// Modified by: 
// Created:     11/23/05 16:17:15
// RCS-ID:      
// Copyright:   CAEN S.p.A. All rights reserved
// Licence:     
/////////////////////////////////////////////////////////////////////////////


#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "appsettings.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include <wx/arrimpl.cpp> 
#include <wx/config.h>			
#include <wx/confbase.h>		
#include <wx/fileconf.h>		
#include <wx/filename.h>
#include <wx/dir.h>


#include "appsettings.h"
#include "V1724_board.h"

#ifndef CAEN_VME_INVALID_HANDLE
	#define CAEN_VME_INVALID_HANDLE			-1
#endif

AppSettings::AppSettings( void (* scope_refresh)( bool)): m_vme_link(0), m_vme_board_num(0), m_vme_handle( CAEN_VME_INVALID_HANDLE),
								m_grid_pen(NULL), m_mid_grid_pen( NULL), m_back_brush(NULL), ScopeRefresh( scope_refresh), m_mutex( wxMUTEX_RECURSIVE)
{

	wxString setting_filename= wxGetCwd()+ wxFileName::GetPathSeparator()+ _("settings.txt");

	wxConfigBase::Set( new wxFileConfig( wxEmptyString, wxEmptyString, setting_filename, setting_filename, wxCONFIG_USE_LOCAL_FILE ));
	
}

AppSettings::~AppSettings(void)
{
	delete wxConfigBase::Set((wxConfigBase *) NULL);

	// Delete channel list
	for( size_t i= 0; i< this->m_board_array.GetCount(); i++)
	{
		delete (GenericBoard*)this->m_board_array[i];
	}
	this->m_board_array.Clear();

	// Close VME handling
	this->EndVME();

	if( this->m_grid_pen!= NULL)
	{
		delete this->m_grid_pen;
		this->m_grid_pen= NULL;
	}
	if( this->m_mid_grid_pen!= NULL)
	{
		delete this->m_mid_grid_pen;
		this->m_mid_grid_pen= NULL;
	}
	if( this->m_back_brush!= NULL)
	{	
		delete this->m_back_brush;
		this->m_back_brush= NULL;
	}
}
/*!
* Load data
*/
bool AppSettings::Load( void)
{
	/*
	Application setting file structure
	[RECORD]
	|
	|- RECORD_FOLDER= ./record/           // The folder where to store record files
	[TRIGGER]
	|
	|- TRIGGER_MSK= 0                     // Bitmask : 0x0001: external 0x0002: Channels 0x0004: autoretrig 
	|- POST_TRIGGER= 0       
	|- TRIGGER_POSITION_KEEP= 0       
	|- SAMPLE_BLOCK_SIZE= 0                    
    |- TRIGGER_EDGE= 0               // 0: No Trigger  1: Rising edge  2: Falling edge
	[MISC]
	|
	|- VME_BOARD_TYPE=V1718
	|- VME_LINK=0
	|- VME_BOARD_NUM=0
	|- GRID_LINE_COLOR= 0xffffff					// RGB color
	|- GRID_LINE_TYPE= 0
	|- GRID_LINE_WIDTH= 1
	|- OFFSET_SEC= 0.0
	|- EXT_RATE_MHZ= 100000000						// external sample rate (MHz)
	|- INT_RATE_MHZ= 100000000						// internal sample rate (MHz)
	|- USE_EXT_CLOCK= 0								// use external clock
	|- USE_TTL= 0									// use TTL/NIM external signals
	|- SEC_2_DIV= 1000								// Div/ Sec conversion factor ( div_val= SEC_2_DIV* sec_val)
	|- MAX_LOG_X= 1000
	|- BACKGROUND_COLOR= 0x000000					// RGB color
	|- ACQ_BOARD_NUM= N
	[ACQ_BOARDS]
	|
	|- [0]
	|   |
	|   |- BOARD_TYPE= V1724
	|   |- { OTHER BOARD DATA}
	|   ...
	|- [N- 1]
	|   |
	|   |- BOARD_TYPE= V1724
	|   |- { OTHER BOARD DATA}
	*/
	unsigned int i;
	// Delete previously stored boards
	this->m_board_array.Clear();
	wxConfigBase* config= wxConfigBase::Get( false);
	if( !config)
	{
		wxLogError( _("Cannot get configuration file\n"));
		return false;
	}

	wxMutexLocker lock( this->m_mutex);
	
	//
	// Get Record Folder
	config->Read(  _("/RECORD/RECORD_FOLDER"), &this->m_record_folder, "./");
	// Sanity checks
	if( !wxDir::Exists( this->m_record_folder))
	{
		// Set to current directory
		this->m_record_folder= "./";
	}

	//
	// Get Trigger msk
	int trigger_msk= 0;
	config->Read( _("/TRIGGER/TRIGGER_MSK"), &trigger_msk, 0);
	this->m_trigger_msk= (AppSettings::TRIGGER_MSK)trigger_msk;

	//
	// Get Post Trigger
	this->m_post_trigger= 0;
	config->Read( _("/TRIGGER/POST_TRIGGER"), &this->m_post_trigger, 0);

	//
	// Get Keep trigger position flags
	int tmp;
	config->Read( _("/TRIGGER/TRIGGER_POSITION_KEEP"), &tmp, 0);
	this->m_trigger_position_keep= tmp? true: false;


	//
	// Get trigger edge type
	int edge_type;
	config->Read( _("/TRIGGER/TRIGGER_EDGE"), &edge_type, (int)NO_TRIGGER);
	this->m_trigger_edge_type= (TRIGGER_EDGE_TYPE)edge_type;

	//
	// Get Sample block size
	this->m_sample_block_size= 0;
	config->Read( _("/TRIGGER/SAMPLE_BLOCK_SIZE"), &this->m_sample_block_size, 0);

	//
	// Get VME board type
	config->Read( _("/MISC/VME_BOARD_TYPE"), &this->m_vme_board_type_string, _("V1718"));
	//
	// Get VME board link
	config->Read( _("/MISC/VME_LINK"), &this->m_vme_link, 0);
	//
	// Get VME board number
	config->Read( _("/MISC/VME_BOARD_NUM"), &this->m_vme_board_num, 0);
	//
	// Get grid line color
	wxString grid_line_color_rgb_string= _("");
	config->Read( _("/MISC/GRID_LINE_COLOR"), &grid_line_color_rgb_string, _("0xffffff"));
	unsigned long grid_line_color_rgb;
	wxString hex_part;
	if( grid_line_color_rgb_string.Upper().StartsWith(_("0X"), &hex_part))
	{
		// Hex format
		if( !hex_part.Trim().ToULong( &grid_line_color_rgb, 16))
		{
			grid_line_color_rgb= 0x00ffffff;
		}
	}
	else
	{
		// Decimal format
		if( !grid_line_color_rgb_string.Trim().ToULong( &grid_line_color_rgb, 10))
		{
			grid_line_color_rgb= 0x00ffffff;
		}
	}
	this->m_grid_line_color.Set( (grid_line_color_rgb>> 16)& 0xff, (grid_line_color_rgb>> 8)& 0xff, grid_line_color_rgb& 0xff);

	//
	// Get line type
	int grid_line_type;
	config->Read( _("/MISC/GRID_LINE_TYPE"), &grid_line_type, 1);
	this->m_grid_line_type= (GRID_LINE_TYPE)grid_line_type;

	//
	// Get line width
	config->Read( _("/MISC/GRID_LINE_WIDTH"), &this->m_grid_line_width, 1);

	//
	// Get horizontal offset (sec)
	config->Read( _("/MISC/OFFSET_SEC"), &this->m_offset_sec, 0.0);

	//
	// Get internal sample rate (MHz)
	config->Read( _("/MISC/INT_RATE_MHZ"), &this->m_int_rate_MHz, 100000000);
	//
	// Get external sample rate (MHz)
	config->Read( _("/MISC/EXT_RATE_MHZ"), &this->m_ext_rate_MHz, 100000000);
	//
	// Get clock type usage (internal or external)
	config->Read( _("/MISC/USE_EXT_CLOCK"), &tmp, 0);
	this->m_use_ext_clock= tmp? true: false;
	
	this->SetClock( this->m_use_ext_clock, this->m_ext_rate_MHz);

	//
	// Get clock type usage (internal or external)
	config->Read( _("/MISC/USE_TTL"), &tmp, 0);
	this->m_use_TTL= tmp? true: false;

	//
	// Get Div/ Sec conversion factor
	config->Read( _("/MISC/SEC_2_DIV"), &this->m_sec_2_div, 1000.0);
	this->SetSec2Div( this->m_sec_2_div);

	//
	// Get maximum logical value X
	config->Read( _("/MISC/MAX_LOG_X"), &this->m_max_log_X, 1024);

	//
	// Get background color
	wxString back_color_rgb_string= _("");
	config->Read( _("/MISC/BACKGROUND_COLOR"), &back_color_rgb_string, _("0x000000"));
	unsigned long back_color_rgb;
	if( back_color_rgb_string.Upper().StartsWith(_("0X"), &hex_part))
	{
		// Hex format
		if( !hex_part.Trim().ToULong( &back_color_rgb, 16))
		{
			back_color_rgb= 0x00ffffff;
		}
	}
	else
	{
		// Decimal format
		if( !back_color_rgb_string.Trim().ToULong( &back_color_rgb, 10))
		{
			back_color_rgb= 0x00ffffff;
		}
	}
	this->m_background_color.Set( (back_color_rgb>> 16)& 0xff, (back_color_rgb>> 8)& 0xff, back_color_rgb& 0xff);

	int acq_board_num= 0;
	config->Read( _("/MISC/ACQ_BOARD_NUM"), &acq_board_num, 0);

	this->InitVME();
	wxString board_base_string= _("/ACQ_BOARDS/");
	//
	// Boards loop
	for( i= 0; i< (unsigned int)acq_board_num; i++)
	{
		wxString board_string= wxString::Format( "%s%i/", board_base_string.c_str(), i);
		//
		// Get the board type
		wxString board_type= _("");
		config->Read( board_string+ _("BOARD_TYPE"), &board_type, _("V1724"));
		if( !board_type.Trim().CmpNoCase( _("V1724")))
		{
			// V1724
			this->m_board_array.Add( new V1724Board( i, this->ScopeRefresh, &this->m_mutex, this));
		}
		else
		{
			// Unknown board type
			wxLogError( wxString::Format( "Unknown board type '%s'\n", board_type.c_str()));
			// return false;
		}

		// Open board handlers
		if( !((GenericBoard*)this->m_board_array[i])->Open( this->m_vme_handle ))
		{
			wxLogError( _("Board open failure\n"));
			//return false;
		}
		// Load board section
		if( !((GenericBoard*)this->m_board_array[i])->LoadConfig( config, board_string))
		{
			wxLogError( _("Error loading board settings\n"));
			//return false;
		}
		// Init board section
		if( !((GenericBoard*)this->m_board_array[i])->Init( ))
		{
			wxLogError( _("Board init failure\n"));
			//return false;
		}
	}

	return true;
}

/*!
* save data
*/
bool AppSettings::Save( void)
{
	wxConfigBase* config= wxConfigBase::Get( false);
	if( !config)
		return false;
	wxMutexLocker lock( this->m_mutex);

	//
	// Delete groups
	wxString group_id;
	long group_index;
	while( config->GetFirstGroup( group_id, group_index))
	{
		config->DeleteGroup( group_id);
	}
	//
	// Set Record Folder
	if( !config->Write( _("/RECORD/RECORD_FOLDER"), this->m_record_folder))
		return false;

	//
	// Set Trigger msk
	if( !config->Write( _("/TRIGGER/TRIGGER_MSK"), (int)this->m_trigger_msk))
		return false;

	//
	// Set Post Trigger
	if( !config->Write( _("/TRIGGER/POST_TRIGGER"), this->m_post_trigger))
		return false;

	//
	// Set Keep trigger position flags
	config->Write( _("/TRIGGER/TRIGGER_POSITION_KEEP"), this->m_trigger_position_keep? 1: 0);

	//
	// Set trigger edge type
	if( !config->Write( _("/TRIGGER/TRIGGER_EDGE"), (int)this->m_trigger_edge_type))
		return false;

	//
	// Set Sample buffer size
	if( !config->Write( _("/TRIGGER/SAMPLE_BLOCK_SIZE"), this->m_sample_block_size))
		return false;

	//
	// Set VME board type
	if( !config->Write( _("/MISC/VME_BOARD_TYPE"), this->m_vme_board_type_string))
		return false;
	//
	// Set VME board link
	if( !config->Write( _("/MISC/VME_LINK"), this->m_vme_link))
		return false;
	//
	// Set VME board number
	if( !config->Write( _("/MISC/VME_BOARD_NUM"), this->m_vme_board_num))
		return false;
	//
	// Set grid line color
	wxString grid_line_color_rgb_string= wxString::Format( "0x%02x%02x%02x", this->m_grid_line_color.Red(), this->m_grid_line_color.Green(), this->m_grid_line_color.Blue());
	if( !config->Write( _("/MISC/GRID_LINE_COLOR"), grid_line_color_rgb_string))
		return false;

	//
	// Set line type
	if( !config->Write( _("/MISC/GRID_LINE_TYPE"), (int)this->m_grid_line_type))
		return false;

	//
	// Set line width
	if( !config->Write( _("/MISC/GRID_LINE_WIDTH"), this->m_grid_line_width))
		return false;

	//
	// Set horizontal offset (sec)
	if( !config->Write( _("/MISC/OFFSET_SEC"), this->m_offset_sec))
		return false;

	//
	// Set internal sample rate (MHz)
	if( !config->Write( _("/MISC/INT_RATE_MHZ"), this->m_int_rate_MHz))
		return false;

	//
	// Set external sample rate (MHz)
	if( !config->Write( _("/MISC/EXT_RATE_MHZ"), this->m_ext_rate_MHz))
		return false;
	//
	// Set clock type usage (internal or external)
	if( !config->Write( _("/MISC/USE_EXT_CLOCK"), this->m_use_ext_clock? 1: 0))
		return false;

	//
	// Set external signals standard: TTL/NIM
	if( !config->Write( _("/MISC/USE_TTL"), this->m_use_TTL? 1: 0))
		return false;
	
	//
	// Set Div/ Sec conversion factor
	if( !config->Write( _("/MISC/SEC_2_DIV"), this->m_sec_2_div))
		return false;

	//
	// Set maximum logical value X
	if( !config->Write( _("/MISC/MAX_LOG_X"), this->m_max_log_X))
		return false;

	//
	// Set grid line color
	wxString back_color_rgb_string= wxString::Format( "0x%02x%02x%02x", this->m_background_color.Red(), this->m_background_color.Green(), this->m_background_color.Blue());
	if( !config->Write( _("/MISC/BACKGROUND_COLOR"), back_color_rgb_string))
		return false;

	//
	// Set board number
	if( !config->Write( _("/MISC/ACQ_BOARD_NUM"), (int)this->m_board_array.GetCount()))
		return false;

	wxString board_base_string= _("/ACQ_BOARDS/");
	//
	// Boards loop
	for( size_t i= 0; i< this->m_board_array.GetCount(); i++)
	{
		wxString board_string= wxString::Format( "%s%i/", board_base_string.c_str(), i);
		//
		// Set the board type
		if( !config->Write( board_string+ _("BOARD_TYPE"), ((GenericBoard*)this->m_board_array[i])->GetType()))
			return false;

		// Save board section
		if( !((GenericBoard*)this->m_board_array[i])->SaveConfig( config, board_string))
			return false;
	}

	if( !config->Flush())
		return false;

	return true;
}
bool AppSettings::InitVME( void)
{
	// Init CAEN VME handle
	CVBoardTypes vme_board_type;
	if( !this->m_vme_board_type_string.CmpNoCase(_("V1718")))
	{
		vme_board_type= cvV1718;
	}
	else if( !this->m_vme_board_type_string.CmpNoCase(_("V2718")))
	{
		vme_board_type= cvV2718;
	}
	else
	{
		wxLogError( wxString::Format( _("Unhandled board type: %s"), this->m_vme_board_type_string.c_str()));
		return false;
	}
	CVErrorCodes cv_error_code;
	if( ( cv_error_code= CAENVME_Init( vme_board_type, this->m_vme_link, this->m_vme_board_num, &this->m_vme_handle))!= cvSuccess)
	{
		wxLogError( wxString::Format( _("VME Init failure board type= %s link= %i board num= %i error= %s"), this->m_vme_board_type_string.c_str(), this->m_vme_link, this->m_vme_board_num, CAENVME_DecodeError( cv_error_code) ));
		return false;
	}
	return true;
}
bool AppSettings::EndVME( void)
{
	CVErrorCodes cv_error_code;
	if( ( cv_error_code= CAENVME_End( this->m_vme_handle))!= cvSuccess)
	{
		wxLogError( wxString::Format( _("VME End failure board type= %s link= %i board num= %i error= %s\n"), this->m_vme_board_type_string.c_str(), this->m_vme_link, this->m_vme_board_num, CAENVME_DecodeError( cv_error_code)));
		return false;
	}		
	this->m_vme_handle= CAEN_VME_INVALID_HANDLE;
	return true;
}

bool AppSettings::SetGridPen( const wxPen* grid_pen, const wxPen* mid_grid_pen)
{
	if( this->m_grid_pen!= NULL)
	{
		delete this->m_grid_pen;
		this->m_grid_pen= NULL;
	}
	this->m_grid_pen= ( grid_pen== NULL)? NULL: new wxPen( *grid_pen);

	if( this->m_mid_grid_pen!= NULL)
	{
		delete this->m_mid_grid_pen;
		this->m_mid_grid_pen= NULL;
	}
	if( mid_grid_pen== NULL)
	{
		this->m_mid_grid_pen= ( grid_pen== NULL)? NULL: new wxPen( *grid_pen);
	}
	else
	{
		this->m_mid_grid_pen= new wxPen( *mid_grid_pen);
	}
	return true;
}

bool AppSettings::SetBackBrush( const wxBrush* back_brush)
{
	if( this->m_back_brush!= NULL)
	{	
		delete this->m_back_brush;
		this->m_back_brush= NULL;
	}
	this->m_back_brush= ( back_brush== NULL)? NULL: new wxBrush( *back_brush);
	return true;
}

void AppSettings::SetDiv2Pix( double div_2_pix_X)
{
	wxMutexLocker lock( this->m_mutex);

	this->m_div_2_pix_X= div_2_pix_X;
	this->m_sec_2_pix= this->m_sec_2_div* this->m_div_2_pix_X;
}

void AppSettings::SetSec2Div( double sec_2_div)
{
	if( sec_2_div< 0.0000001)
		sec_2_div= 0.0000001;
	this->m_sec_2_div= sec_2_div;

	this->m_sec_2_pix= this->m_sec_2_div* this->m_div_2_pix_X;
}

void AppSettings::SetSec2Sample( double sec_2_sample)
{
	if( sec_2_sample< 0.0000000001)
		sec_2_sample= 0.0000000001;
	this->m_sec_2_sample= sec_2_sample;

	this->m_sample_2_sec= 1/ this->m_sec_2_sample;
}
void AppSettings::SetClock( bool use_ext_clock, int ext_rate_MHz)
{
	if( ext_rate_MHz< 1)
		ext_rate_MHz= 1;
	else if( ext_rate_MHz> 100)
		ext_rate_MHz= 100;
	this->m_ext_rate_MHz= ext_rate_MHz;

	this->m_use_ext_clock= use_ext_clock;

	this->SetSec2Sample( this->m_use_ext_clock? (( double) ext_rate_MHz)* 1000000.0: ((double)this->m_int_rate_MHz)* 1000000.0);
}
