/////////////////////////////////////////////////////////////////////////////
// Name:        appsettings.h
// Purpose:     
// Author:      NDA
// Modified by: 
// Created:     11/23/05 16:17:15
// RCS-ID:      
// Copyright:   CAEN S.p.A. All rights reserved
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _APPSETTINGS_H_
#define _APPSETTINGS_H_

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma interface "appsettings.cpp"
#endif

#include <wx/thread.h>
#include "generic_board.h"

class wxMutex;

class AppSettings
{
public:
	typedef enum
	{
		EXT_TRIGGER_MSK=		0x0001,				// external trigger
		CHANNEL_TRIGGER_MSK=	0x0002,				// Channels trigger
		SW_AUTO_TRIGGER_MSK=	0x0004,				// Software auto retrigger
	} TRIGGER_MSK;
	typedef enum
	{
		NO_TRIGGER= 0,
		RAISING_EDGE,
		FALLING_EDGE,
	} TRIGGER_EDGE_TYPE;
	typedef enum
	{
		NO_LINE= 0,
		SOLID,
		DASHED,
		DASH_DOT,
		DOTTED,
	} GRID_LINE_TYPE;

public:
	//
	// Methods
	AppSettings( void (* scope_refresh)( bool));
	~AppSettings( void);
	bool Load( void);
	bool Save( void);

	//
	// Common lock
	wxMutex m_mutex;
	//
	// Properties
	TRIGGER_MSK m_trigger_msk;					// Trigger type masks
	int m_post_trigger;							// Number of post trigger samples ( < 0 means pretrigger)
	int m_sample_block_size;					// The size of samples' buffer
	wxArrayPtrVoid m_board_array;				// Boards array
	wxColor m_grid_line_color;					// The grid color
	GRID_LINE_TYPE m_grid_line_type;			// The grid line type
	int m_grid_line_width;						// The grid line width
	wxColor m_background_color;					// The background color
	wxString m_record_folder;					// The folder where to store recorded files
	TRIGGER_EDGE_TYPE m_trigger_edge_type;		// Trigger edge type

	wxPen *m_grid_pen;
	wxPen *m_mid_grid_pen;
	wxBrush *m_back_brush;
	int m_max_log_X;							// maximum logical unit X
	double m_offset_sec;						// Horizontal offset (sec)
	double m_sec_2_sample;						// Samples/ Sec conversion factor	
	double m_sec_2_div;							// Div/ Sec conversion factor

	double m_sample_2_sec;						// Sec/ Sample conversion factor	
	int m_int_rate_MHz;							// Internal clock (MHz)
	int m_ext_rate_MHz;							// External clock (MHz)
	bool m_use_ext_clock;						// External/ Internal clock usage
	bool m_use_TTL;								// Use TTL/NIM external signals 
	bool m_trigger_position_keep;				// Keep trigger position on screen

	double m_sec_2_pix;
	double m_div_2_pix_X, m_div_2_pix_Y;
	int m_pix_Y, m_pix_X;

	bool SetGridPen( const wxPen* grid_pen, const wxPen* mid_grid_pen= NULL);
	bool SetBackBrush( const wxBrush* back_brush);

	void (* ScopeRefresh)(bool);

	void SetDiv2Pix( double div_2_pix_X);
	void SetSec2Div( double sec_2_div);
	void SetSec2Sample( double sec_2_sample);
	void SetClock( bool use_ext_clock, int ext_rate_MHz);

	inline UINT32 Sec2Pix( double value){ return (UINT32)(double)( this->m_sec_2_pix* value);};
	inline double Sample2Sec( int value){ return this->m_sample_2_sec* (double)value;};

protected:
	bool EndVME( void);
	bool InitVME( void);

	wxString m_vme_board_type_string;
	int m_vme_link;
	int m_vme_board_num;
	int32_t m_vme_handle;
};


#endif


