/////////////////////////////////////////////////////////////////////////////
// Name:        generic_board.h
// Purpose:     
// Author:      NDA
// Modified by: 
// Created:     04/03/06 14:18:40
// RCS-ID:      
// Copyright:   CAEN S.p.A All rights reserved.
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _GENERIC_BOARD_H_
#define _GENERIC_BOARD_H_

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma interface "generic_board.cpp"
#endif

/*!
 * Includes
 */
#include <wx/arrimpl.cpp> 
extern "C" 
{
	#include "CAENVMETool/cvt_board_commons.h"
}
#include "generic_board_channel.h"

class BoardWorkerThread;
class AppSettings;

class GenericBoard
{
public:
	GenericBoard( int board_index, void (* scope_refresh)( bool), wxMutex *p_data_mutex, AppSettings *p_app_settings);
	virtual ~GenericBoard(void);

	//
	// Properties
	wxArrayPtrVoid m_channel_array;
	wxMutex *m_p_data_mutex;
	void (* ScopeRefresh)(bool);
	
	//
	// Methods
	virtual bool LoadConfig( wxConfigBase* p_config, const wxString& base_section);
	virtual bool SaveConfig( wxConfigBase* p_config, const wxString& base_section);

	// 
	// Pure virtual methods
	virtual wxString GetType()= 0;
	virtual bool Open( long vme_handle)= 0;
	virtual GenericBoardChannel* CreateChannel( int ch_index, void (* scope_refresh)( bool), wxMutex *p_data_mutex)= 0;
	virtual bool WriteSampleBufferSize( void) = 0;
	virtual bool WriteTriggerMode( bool started) = 0;
	virtual bool WriteSoftwareTrigger( void) = 0;
	virtual bool SetFrontPanelIO( bool use_TTL)= 0;

	virtual bool Init( void){ return true;};

	void DrawTrigger( wxDC &dc);
	void DrawCursor( wxDC &dc);
	void DrawSamples( wxDC &dc);
	void SetDiv2Pix( double div_2_pix_Y);
	void SetLeftDiv2Pix( double div_2_pix_Y);
	void SetSec2Div( double sec_2_div);
	void SetPix( int pix_X, int pix_Y);
	void SetLeftPix( int pix_X, int pix_Y);
	void DrawPosition( wxDC &dc);

	bool DoStartRunning( void);
	void DoStopRunning( void);
	bool ToggleRecordStatus( const wxString &path, const wxString &file_name, const wxString &timestamp, bool start);
	bool IsRunning( void);
    void SetRunning( bool value);
    bool m_thread_exited;
protected:
	cvt_board_data* m_p_data;
	BoardWorkerThread *m_p_worker_thread;
	bool m_is_running;
	AppSettings *m_p_app_settings;
	int m_board_index;
	bool m_is_recording;
friend class BoardWorkerThread;
};


#endif		// _GENERIC_BOARD_H_

