/////////////////////////////////////////////////////////////////////////////
// Name:        generic_board_channel.cpp
// Purpose:     
// Author:      NDA
// Modified by: 
// Created:     04/03/06 14:18:40
// RCS-ID:      
// Copyright:   CAEN S.p.A. All rights reserved
// Licence:     
/////////////////////////////////////////////////////////////////////////////


#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "generic_board_channel.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include <wx/filename.h>


#include "generic_board_channel.h"
#include "common_ch_control.h"
#include "cursor_ch_control.h"
#include "appsettings.h"
#include "drawing_panel.h"

GenericBoardChannel::GenericBoardChannel( int ch_index, cvt_board_data* p_data, void ( *scope_refresh)( bool), wxMutex* p_data_mutex, AppSettings *p_app_settings):
		m_line_pen(NULL), m_trigger_pen(NULL), ScopeRefresh( scope_refresh), m_p_data_mutex( p_data_mutex),
		m_p_data( p_data), m_ch_index( ch_index), m_p_app_settings( p_app_settings), m_p_record_file( NULL), m_trigger_brush( NULL),
        m_samples_mutex( wxMUTEX_RECURSIVE), m_record_mutex( wxMUTEX_RECURSIVE)
{
	this->m_sample_buffer_size= 1024;
	this->m_p_sample_buffer= new UINT16[ this->m_sample_buffer_size];
	this->m_sample_buffer_count= 0;

	this->m_tmp_sample_buffer_size= 1024;
	this->m_p_tmp_sample_buffer= new UINT16[ this->m_tmp_sample_buffer_size];
	this->m_tmp_sample_buffer_count= 0;

	this->m_volt_2_div= 1.0;
	this->m_is_recording= false;

	this->m_medium_value= 0;
	this->m_overload_up= false;
	this->m_overload_down= false;
	this->m_cursor_position= 0;
	this->m_cursor_enabled= false;

}

GenericBoardChannel::~GenericBoardChannel(void)
{
	if( this->m_line_pen!= NULL)
	{
		delete this->m_line_pen;
		this->m_line_pen= NULL;
	}

	if( this->m_trigger_pen!= NULL)
	{
		delete this->m_trigger_pen;
		this->m_trigger_pen= NULL;
	}
	if( this->m_trigger_brush!= NULL)
	{
		delete this->m_trigger_brush;
		this->m_trigger_brush= NULL;
	}
	if( this->m_p_sample_buffer)
	{
		delete this->m_p_sample_buffer;
		this->m_p_sample_buffer= NULL;
		this->m_sample_buffer_count= 0;
	}
	if( this->m_p_tmp_sample_buffer)
	{
		delete this->m_p_tmp_sample_buffer;
		this->m_p_tmp_sample_buffer= NULL;
		this->m_tmp_sample_buffer_count= 0;
	}
	if( this->m_p_record_file)
	{
		delete this->m_p_record_file;
		this->m_p_record_file= NULL;
	}
}

bool GenericBoardChannel::LoadConfig( wxConfigBase* p_config, const wxString& base_section)
{
	/*
		Application setting channel parameters' file structure

          |- ENABLED= 1
          |- LINE_COLOR= 0xaaff00          // RGB color
          |- LINE_WIDTH= 1                 // Pixels
          |- OFFSET_VOLT= 0.0              // Volts
          |- VOLT_2_BIT= 26214.4           // Bits/ Volt conversion factor ( bit_val= VOLT_2_BIT* volt_val) (2^16/2.500)
          |- VOLT_2_DIV= 4                 // Div/ Volts conversion factor ( div_val= VOLT_2_DIV* volt_val)
          |- DAC_VOLT_2_BIT=26214.4        // DAC Bits/ Volt conversion factor ( bit_val= DAC_VOLT_2_BIT* volt_val) (2^16/2.500)
          |- DAC_OFFSET_BIT=0              // DAC Offset Bit
          |- TRIGGER_THRESHOLD_VOLT= 1.25  // Trigger Threshold (Volts)
          |- TRIGGER_THR_SAMPLE= 100       // Number of samples after trigger threshold
          |- RECORD_RATE_DIVIDER= 1        // Samples' recording rate divider
          |- RECORD_ENABLED= 1             // Samples' recording enabled
          |- RECORD_FORMAT= 0              // Record save format 0: hex 1: ASCII decimal 2: ASCII hexdecimal
	*/

	//
	// Get channel enable bit
	this->m_enabled= FALSE;
	p_config->Read( base_section+ _("ENABLED"), &this->m_enabled, 0);

	//
	// Get line color
	wxString line_color_rgb_string= _("");
	p_config->Read( base_section+ _("LINE_COLOR"), &line_color_rgb_string, _("0xffffff"));
	unsigned long line_color_rgb;
	wxString hex_part;
	if( line_color_rgb_string.Upper().StartsWith(_("0X"), &hex_part))
	{
		// Hex format
		if( !hex_part.Trim().ToULong( &line_color_rgb, 16))
		{
			line_color_rgb= 0x00ffffff;
		}
	}
	else
	{
		// Decimal format
		if( !line_color_rgb_string.Trim().ToULong( &line_color_rgb, 10))
		{
			line_color_rgb= 0x00ffffff;
		}
	}
	this->m_line_color.Set( (line_color_rgb>> 16)& 0xff, (line_color_rgb>> 8)& 0xff, line_color_rgb& 0xff);
	//
	// Get line width
	this->m_line_width= 1;
	p_config->Read( base_section+ _("LINE_WIDTH"), &this->m_line_width, 1);
	//
	// Get offset_volt
	this->m_offset_y= 0;
	p_config->Read( base_section+ _("OFFSET_Y"), &this->m_offset_y, 0.0);
	//
	// Get volt_2_bit
	this->m_bit_2_volt= 1.0;
	this->m_volt_2_bit= 1.0;
	p_config->Read( base_section+ _("VOLT_2_BIT"), &this->m_volt_2_bit, 26214.4);
	if( this->m_volt_2_bit== 0)
		this->m_volt_2_bit= 1.0;
	this->m_bit_2_volt= 1/ this->m_volt_2_bit;
	//
	// Get div_2_volt
	double value= 1.0;
	p_config->Read( base_section+ _("VOLT_2_DIV"), &value, 4.0);
	this->SetVolt2Div( value);

	//
	// Get DAC offset volt
	this->m_DAC_offset_bit= 0x8000;
	p_config->Read( base_section+ _("DAC_OFFSET_BIT"), &this->m_DAC_offset_bit, 0x8000);

	//
	// Get DAC bit 2 volt
	this->m_DAC_volt_2_bit= 26214.4;
	p_config->Read( base_section+ _("DAC_VOLT_2_BIT"), &this->m_DAC_volt_2_bit, 26214.4);

	//
	// Get trigger threshold
	this->m_trigger_threshold_volt= 0;
	p_config->Read( base_section+ _("TRIGGER_THRESHOLD_VOLT"), &this->m_trigger_threshold_volt, 0.0);
	//
	// Get trigger threshold samples
	this->m_trigger_thr_sample= 0;
	p_config->Read( base_section+ _("TRIGGER_THR_SAMPLE"), &this->m_trigger_thr_sample, 0);
	//
	// Get record rate divider
	this->m_record_rate_divider= 1;
	p_config->Read( base_section+ _("RECORD_RATE_DIVIDER"), &this->m_record_rate_divider, 1);
	if( this->m_record_rate_divider<= 0)
	{
		this->m_record_rate_divider= 1;
	}
	//
	// Get record enable bit
	this->m_record_enabled= FALSE;
	p_config->Read( base_section+ _("RECORD_ENABLED"), &this->m_record_enabled, 0);
	//
	// Get record format type
	int record_format_type;
	p_config->Read( base_section+ _("RECORD_FORMAT"), &record_format_type, (int)GenericBoardChannel::HEX);
	this->m_record_format_type= (GenericBoardChannel::RECORD_FORMAT_TYPE)record_format_type;

	return true;

}
bool GenericBoardChannel::SaveConfig( wxConfigBase* p_config, const wxString& base_section)
{

	//
	// Set channel enable bit
	if( !p_config->Write( base_section+ _("ENABLED"), this->m_enabled? 1: 0))
		return false;
	//
	// Set line color
	wxString line_color_rgb_string= wxString::Format( "0x%02x%02x%02x", this->m_line_color.Red(), this->m_line_color.Green(), this->m_line_color.Blue());
	if( !p_config->Write( base_section+ _("LINE_COLOR"), line_color_rgb_string))
		return false;
	//
	// Set line width
	if( !p_config->Write( base_section+ _("LINE_WIDTH"), this->m_line_width))
		return false;
	//
	// Set offset_volt
	if( !p_config->Write( base_section+ _("OFFSET_Y"), this->m_offset_y))
		return false;
	//
	// Set volt_2_bit
	if( !p_config->Write( base_section+ _("VOLT_2_BIT"), this->m_volt_2_bit))
		return false;
	//
	// Set volt_2_div
	if( !p_config->Write( base_section+ _("VOLT_2_DIV"), this->m_volt_2_div))
		return false;

	//
	// Set DAC offset volt
	if( !p_config->Write( base_section+ _("DAC_OFFSET_BIT"), this->m_DAC_offset_bit))
		return false;

	//
	// Set DAC bit 2 volt
	if( !p_config->Write( base_section+ _("DAC_VOLT_2_BIT"), this->m_DAC_volt_2_bit))
		return false;

	//
	// Set trigger threshold
	if( !p_config->Write( base_section+ _("TRIGGER_THRESHOLD_VOLT"), this->m_trigger_threshold_volt))
		return false;
	//
	// Set trigger threshold samples
	if( !p_config->Write( base_section+ _("TRIGGER_THR_SAMPLE"), this->m_trigger_thr_sample))
		return false;
	//
	// Set record rate divider
	if( this->m_record_rate_divider<= 0)
	{
		this->m_record_rate_divider= 1;
	}
	if( !p_config->Write( base_section+ _("RECORD_RATE_DIVIDER"), this->m_record_rate_divider))
		return false;
	//
	// Set record enable bit
	if( !p_config->Write( base_section+ _("RECORD_ENABLED"), this->m_record_enabled? 1: 0))
		return false;

	//
	// Set record format type
	if( !p_config->Write( base_section+ _("RECORD_FORMAT"), (int)this->m_record_format_type))
		return false;
	return true;
}

bool GenericBoardChannel::SetLinePen( const wxPen* line_pen, const wxPen* trigger_pen)
{
	if( this->m_line_pen!= NULL)
	{
		delete this->m_line_pen;
		this->m_line_pen= NULL;
	}
	this->m_line_pen= ( line_pen== NULL)? NULL: new wxPen( *line_pen);

	if( this->m_trigger_pen!= NULL)
	{
		delete this->m_trigger_pen;
		this->m_trigger_pen= NULL;
	}
	if( this->m_trigger_brush!= NULL)
	{
		delete this->m_trigger_brush;
		this->m_trigger_brush= NULL;
	}
	if( trigger_pen== NULL)
	{
		this->m_trigger_pen= ( line_pen== NULL)? NULL: new wxPen( *line_pen);
		this->m_trigger_brush= new wxBrush( line_pen->GetColour());
	}
	else
	{
		this->m_trigger_pen= new wxPen( *trigger_pen);
		this->m_trigger_brush= new wxBrush( line_pen->GetColour());
	}

	if( this->ScopeRefresh!= NULL)
		(this->ScopeRefresh)( false);
	return true;
}
void GenericBoardChannel::DrawPosition( wxDC &dc)
{
	wxMutexLocker lock( this->m_p_app_settings->m_mutex);

	const int MARKER_SIZE_PIX_Y= 2;
	const int CANVAS_SIZE_PIX_X= 17;
	const int MARKER_ARROW_PIX_X= 5;
	
	static wxPoint marker[ ] =
	{
		wxPoint( 2, -MARKER_SIZE_PIX_Y),
		wxPoint( CANVAS_SIZE_PIX_X- MARKER_ARROW_PIX_X, -MARKER_SIZE_PIX_Y),
		wxPoint( CANVAS_SIZE_PIX_X, 0),
		wxPoint( CANVAS_SIZE_PIX_X- MARKER_ARROW_PIX_X, +MARKER_SIZE_PIX_Y),
		wxPoint( 2, +MARKER_SIZE_PIX_Y),
	};
	const int marker_points= sizeof( marker)/ sizeof( marker[0]);

	if( !this->m_enabled|| 
		( this->m_trigger_pen== NULL)||
		( this->m_trigger_brush== NULL))
		return;
	double offset_y= this->m_left_div_2_pix_Y*( (double)( DrawingCanvas::NUM_DIV_PER_SCREEN>> 1)+ this->m_offset_y);

	dc.SetBrush( *this->m_trigger_brush);
	dc.SetPen( *this->m_trigger_pen);
	dc.DrawPolygon( marker_points, marker, 0, (int)(double)( this->m_left_pix_Y- offset_y), wxODDEVEN_RULE);
	dc.SetPen( wxNullPen);
	dc.SetBrush( wxNullBrush);
}

void GenericBoardChannel::DrawTrigger( wxDC &dc)
{
	wxMutexLocker lock( this->m_p_app_settings->m_mutex);

	const int TRIGGER_SIZE_PIX_X= 5;
	const int TRIGGER_SIZE_PIX_Y= 3;
	static wxPoint raising_trigger[ ] =
	{
		wxPoint( -TRIGGER_SIZE_PIX_X, +TRIGGER_SIZE_PIX_Y),
		wxPoint( 0, +TRIGGER_SIZE_PIX_Y),
		wxPoint( 0, -TRIGGER_SIZE_PIX_Y),
		wxPoint( +TRIGGER_SIZE_PIX_X, -TRIGGER_SIZE_PIX_Y)
	};
	static wxPoint falling_trigger[ ] =
	{
		wxPoint( -TRIGGER_SIZE_PIX_X, -TRIGGER_SIZE_PIX_Y),
		wxPoint( 0, -TRIGGER_SIZE_PIX_Y),
		wxPoint( 0, +TRIGGER_SIZE_PIX_Y),
		wxPoint( +TRIGGER_SIZE_PIX_X, +TRIGGER_SIZE_PIX_Y)
	};

	if( !this->m_enabled|| 
		( this->m_trigger_pen== NULL))
		return;

	dc.SetPen( *this->m_trigger_pen);

	double offset_x= this->Sample2Sec( this->m_p_app_settings->m_max_log_X- ( this->m_p_app_settings->m_post_trigger+ this->m_trigger_thr_sample))+ this->m_p_app_settings->m_offset_sec;
	//double offset_y= this->m_offset_volt/*+ this->m_trigger_threshold*/;
	double offset_y= this->m_div_2_pix_Y*( (double)( DrawingCanvas::NUM_DIV_PER_SCREEN>> 1)+ this->m_offset_y);
	switch( this->m_p_app_settings->m_trigger_edge_type)
	{
	case AppSettings::FALLING_EDGE:
		//dc.DrawLines( 4, falling_trigger, this->Sec2Pix( offset_x), this->PixY2Dc( this->Volt2Pix( offset_y)));
		dc.DrawLines( 4, falling_trigger, this->Sec2Pix( offset_x), this->PixY2Dc( (int)offset_y));
		break;
	case AppSettings::RAISING_EDGE:
		//dc.DrawLines( 4, raising_trigger, this->Sec2Pix( offset_x), this->PixY2Dc( this->Volt2Pix( offset_y)));
		dc.DrawLines( 4, raising_trigger, this->Sec2Pix( offset_x), this->PixY2Dc( (int)offset_y));
		break;
	default:
		break;
	}
	dc.SetPen( wxNullPen);
}
void GenericBoardChannel::DrawCursor( wxDC &dc)
{
	wxMutexLocker lock( this->m_p_app_settings->m_mutex);
	if( !this->m_enabled|| 
		!this->m_cursor_enabled|| 
		( this->m_trigger_pen== NULL))
		return;
	dc.SetPen( *this->m_trigger_pen);
	int offset_x= this->Sec2Pix( this->m_p_app_settings->m_offset_sec+ this->Sample2Sec( this->m_cursor_position));
	dc.DrawLine( offset_x, 0, offset_x, this->m_pix_Y);
	
	// Restore original pen
	dc.SetPen( wxNullPen);
}

void GenericBoardChannel::DrawSamples( wxDC &dc)
{
	wxMutexLocker lock( this->m_p_app_settings->m_mutex);
#ifdef SAMPLE_LOCK
	wxMutexLocker sample_lock( this->m_samples_mutex);
#endif
	if( !this->m_enabled)
		return;
	if( !this->GetBufferCount())
		return;

	double cursor_value= 0;
	int cursor_pos_x= this->m_cursor_position;
	dc.SetPen( *this->m_line_pen);
	double offset_x= this->m_p_app_settings->m_offset_sec;
	//double offset_y= this->m_offset_volt;
	double offset_y= this->m_div_2_pix_Y*( (double)(int)( DrawingCanvas::NUM_DIV_PER_SCREEN>> 1)+ this->m_offset_y);

	int x0, y0, x1, y1;
	int i= this->Sec2Sample( this->Pix2Sec( 0)- offset_x);
	if( i< 0)
	{
		i= 0;
	}
	else if(( i>= (int)this->GetBufferCount())&& ( i>= this->m_p_app_settings->m_max_log_X))
	{
		i= ( (int)this->GetBufferCount()> this->m_p_app_settings->m_max_log_X)? this->m_p_app_settings->m_max_log_X: this->GetBufferCount();
	}
	double sample_volt= this->GetSampleVolt( i);
	x0= this->Sec2Pix( offset_x+ this->Sample2Sec( i));
	//y0= this->PixY2Dc( this->Volt2Pix( offset_y+ sample_volt));
	y0= this->PixY2Dc( (int)(double)( offset_y+ (double)this->Volt2Pix( sample_volt)));
	bool overload_down= false;
	bool overload_up= false;
    int medium_value_count= 1;
	this->m_medium_value= sample_volt;
	if( this->GetSample(i)>= (0x7ffc))
	{
		overload_up= true;
	}
	else if( this->GetSample(i)<= -0x7ffc)
	{
		overload_down= true;
	}
	if( cursor_pos_x== 0)
	{
		cursor_value= sample_volt* 1000;
	}
	int step= this->Sec2Sample( this->Pix2Sec( 1));
	if( step== 0)
		step= 1;
	i+= step;
	for( ; ( i< (int)this->GetBufferCount())&& ( i< this->m_p_app_settings->m_max_log_X); i+= step)
	{
		if( this->GetSample(i)>= 0x7ffc)
		{
			overload_up= true;
		}
		else if( this->GetSample(i)<= -0x7ffc)
		{
			overload_down= true;
		}
		sample_volt= this->GetSampleVolt( i);
		this->m_medium_value+= sample_volt;
        ++medium_value_count;
		x1= this->Sec2Pix( offset_x+ this->Sample2Sec( i));
		y1= this->PixY2Dc( (int)(double)(offset_y+ (double)this->Volt2Pix( sample_volt)));

		if(( cursor_pos_x>= i)&&
			( cursor_pos_x< i+ step))
		{
			cursor_value= sample_volt* 1000;
		}
		if( x0== x1)
			continue;
		if( x1> this->m_pix_X+ 10)
			break;
		if( x1>= -10)
			dc.DrawLine( x0, y0, x1, y1);
		x0= x1;
		y0= y1;
	}
	dc.SetPen( wxNullPen);
	this->m_medium_value/= medium_value_count;

	// Update controls here
	this->m_p_common_ch_control->m_medium_value_control->SetLabel( wxString::Format( "%.3f", this->m_medium_value));
	if( ( overload_down^ this->m_overload_down) || ( overload_up^ this->m_overload_up))
	{
		this->m_p_common_ch_control->m_overload_control->SetBackgroundColour( ( overload_down|| overload_up)? wxColour(255, 0, 0): wxColour(0, 255, 0));
		this->m_p_common_ch_control->m_overload_control->Refresh();
		this->m_overload_down= overload_down;
		this->m_overload_up= overload_up;
	}
	if( this->m_cursor_enabled)
	{
		this->m_p_cursor_ch_control->m_cursor_value_control->SetLabel( wxString::Format( "%.3f", cursor_value));
	}
}
void GenericBoardChannel::SetVolt2Div( double volt_2_div)
{
	if( volt_2_div< 0.0000001)
		volt_2_div= 0.0000001;
	this->m_volt_2_div= volt_2_div;
	//this->m_div_2_volt= 1/ volt_2_div;

	this->m_volt_2_pix= this->m_volt_2_div* this->m_div_2_pix_Y;
}
inline int GenericBoardChannel::Sec2Pix( double value)
{ 
	return (int)(double)( this->m_p_app_settings->m_sec_2_pix* value);
}
inline double GenericBoardChannel::Sample2Sec( int value)
{ 
	return this->m_p_app_settings->m_sample_2_sec* (double)value;
}
inline double GenericBoardChannel::Pix2Sec( int value)
{ 
	return (double)( value)/ this->m_p_app_settings->m_sec_2_pix;
}
inline int GenericBoardChannel::Sec2Sample( double value)
{ 
	return (int)(double)( (double)value/ this->m_p_app_settings->m_sample_2_sec);
}
void GenericBoardChannel::SetDiv2Pix( double div_2_pix_Y)
{
	wxMutexLocker lock( this->m_p_app_settings->m_mutex);

	this->m_div_2_pix_Y= div_2_pix_Y;
	this->m_volt_2_pix= this->m_volt_2_div* this->m_div_2_pix_Y;	
}
void GenericBoardChannel::SetLeftDiv2Pix( double div_2_pix_Y)
{
	wxMutexLocker lock( this->m_p_app_settings->m_mutex);

	this->m_left_div_2_pix_Y= div_2_pix_Y;
	//this->m_volt_2_pix= this->m_volt_2_div* this->m_div_2_pix_Y;	
}

bool GenericBoardChannel::ToggleRecordStatus( const wxString &path, const wxString &file_name, const wxString &timestamp, bool start)
{
	wxMutexLocker lock( this->m_record_mutex);
	if( start)
	{
		// Start recording request
		if( this->m_is_recording)
			// Already recording
			return true;
		if( !this->m_record_enabled)
			return true;
		// Create recording file
		if( this->m_p_record_file)
		{
			delete this->m_p_record_file;
			this->m_p_record_file= NULL;
		}
		wxString file_ext= _("hex");
		switch( this->m_record_format_type)
		{
		case ASCII_DEC:
			file_ext= _("adc");
			break;
		case ASCII_HEX:
			file_ext= _("ahx");
			break;
		case HEX:
		default:
			this->m_record_format_type= HEX;
			file_ext= _("bhx");
			break;
		}
		this->m_record_sample_counter= 0;
		// Append channel id & timestamp
		wxString new_file_name( wxString::Format( "%sC%02d_%s", file_name.c_str(), this->m_ch_index, timestamp.c_str()));
		wxFileName filename( path, new_file_name, file_ext);
		filename.Normalize();
		this->m_p_record_file= new wxFile( filename.GetFullPath(), wxFile::write_append);
		if( !this->m_p_record_file->IsOpened())
		{
			delete this->m_p_record_file;
			this->m_p_record_file= NULL;
			return false;
		}
	}
	else
	{
		// Stop recording request
		if( !this->m_is_recording)
			// Nothing to do
			return true;

		// Close recording buffer
		if( this->m_p_record_file)
		{
			this->m_p_record_file->Close();
			delete this->m_p_record_file;
			this->m_p_record_file= NULL;
		}
	}
	this->m_is_recording= start;
	return true;
}

bool GenericBoardChannel::RecordSamples( void)
{
	wxMutexLocker lock( this->m_record_mutex);
	if( !this->m_record_enabled)
		return true;
	if( !this->m_p_record_file||
		!this->m_p_sample_buffer)
		return false;

	// Do Save to file ...
	switch( this->m_record_format_type)
	{
	case ASCII_DEC:
		{
			// Text dec format
			wxString line;
			for( UINT32 i= 0; i< this->m_sample_buffer_count; i++)
			{
				if( !this->m_record_sample_counter)
				{
					// record converted data
					line= wxString::Format( "%d\r\n", this->GetSample( i));
					this->m_p_record_file->Write( line);
				}
				if( ++this->m_record_sample_counter>= (UINT32)this->m_record_rate_divider)
				{
					this->m_record_sample_counter= 0;
				}
			}
		}
		break;
	case ASCII_HEX:
		{
			// Text hex format
			wxString line;
			for( UINT32 i= 0; i< this->m_sample_buffer_count; i++)
			{
				if( !this->m_record_sample_counter)
				{
					// record raw data
					line= wxString::Format( "%04X\r\n", this->m_p_sample_buffer[ i]);
					this->m_p_record_file->Write( line);
				}
				if( ++this->m_record_sample_counter>= (UINT32)this->m_record_rate_divider)
				{
					this->m_record_sample_counter= 0;
				}
			}
		}
		break;
	case HEX:
	default:
		{
			// Binary hex format
			for( UINT32 i= 0; i< this->m_sample_buffer_count; i++)
			{
				if( !this->m_record_sample_counter)
				{
					// record raw data
					this->m_p_record_file->Write( &m_p_sample_buffer[ i], sizeof( m_p_sample_buffer[ i]));
				}
				if( ++this->m_record_sample_counter>= (UINT32)this->m_record_rate_divider)
				{
					this->m_record_sample_counter= 0;
				}
			}
		}
		break;
	}

	return true;
}

