/////////////////////////////////////////////////////////////////////////////
// Name:        generic_board_channel.h
// Purpose:     
// Author:      NDA
// Modified by: 
// Created:     04/03/06 14:18:40
// RCS-ID:      
// Copyright:   CAEN S.p.A All rights reserved.
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _GENERIC_BOARD_CHANNEL_H_
#define _GENERIC_BOARD_CHANNEL_H_

#define SAMPLE_LOCK

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma interface "generic_board_channel.cpp"
#endif

/*!
 * Includes
 */
#include <wx/colour.h>
#include <wx/config.h>			
#include <wx/confbase.h>		
#include <wx/fileconf.h>		
#include <wx/file.h>		

#include "common_defs.h"

extern "C" 
{
	#include "CAENVMETool/cvt_board_commons.h"
}

//#include "appsettings.h"
class AppSettings;
class RecordChControl;
class CommonChControl;
class CursorChControl;

class GenericBoardChannel
{
public:
	typedef enum
	{
		HEX= 0,
		ASCII_DEC,
		ASCII_HEX,
	} RECORD_FORMAT_TYPE;
public:
	GenericBoardChannel( int ch_index, cvt_board_data* p_data, void (* scope_refresh)( bool), wxMutex *p_data_mutex, AppSettings *p_app_settings);
	virtual ~GenericBoardChannel(void);
	//
	// Properties
	bool m_enabled;
	wxColour m_line_color;
	int m_line_width;
	int m_DAC_offset_bit;
	double m_DAC_bit_2_volt, m_DAC_volt_2_bit;
	double m_trigger_threshold_volt;
	int m_trigger_thr_sample;
	int m_record_rate_divider;
	bool m_record_enabled;
	RECORD_FORMAT_TYPE m_record_format_type;

	double m_offset_y;
	double m_bit_2_volt;

	double m_volt_2_bit, m_volt_2_div, m_volt_2_pix;
	double m_div_2_pix_X, m_div_2_pix_Y;
	int m_pix_Y, m_pix_X;
	double m_left_div_2_pix_Y;
	int m_left_pix_Y, m_left_pix_X;
	int m_cursor_position;
	bool m_cursor_enabled;

	double m_medium_value;
	bool m_overload_up, m_overload_down;

	wxPen *m_line_pen;
	wxPen *m_trigger_pen;
	wxBrush *m_trigger_brush;

	RecordChControl* m_p_record_ch_control;
	CommonChControl* m_p_common_ch_control;
	CursorChControl* m_p_cursor_ch_control;

	void (* ScopeRefresh)( bool);
	bool SetLinePen( const wxPen* line_pen, const wxPen* trigger_pen= NULL);

	//
	// Methods
	virtual bool LoadConfig( wxConfigBase* p_config, const wxString& base_section);
	virtual bool SaveConfig( wxConfigBase* p_config, const wxString& base_section);

	UINT32 GetBufferCount()	{ return this->m_sample_buffer_count;};
	void SetSec2Pix( double sec_per_div);

	inline int GetSample( int i){ return (int)this->m_p_sample_buffer[i]- 0x8000;};
	inline double GetSampleVolt( int i){ return (double)( this->m_p_sample_buffer[i]- 0x8000)* this->m_bit_2_volt;};
	inline int Volt2Pix( double value){ return (int)(double)( this->m_volt_2_pix* value);};
	inline int PixY2Dc( int value){ return this->m_pix_Y- value;};

	void SetDiv2Pix( double div_2_pix_Y);
	void SetLeftDiv2Pix( double div_2_pix_Y);

	inline int Sec2Pix( double value);
	inline double Sample2Sec( int value);
	inline double Pix2Sec( int value);
	inline int Sec2Sample( double value);

	void SetVolt2Div( double volt_2_div);
	void SetVolt2Pix( double volt_2_pix);
	void SetPix( int pix_X, int pix_Y){ this->m_pix_X= pix_X; this->m_pix_Y= pix_Y;};
	void SetLeftPix( int pix_X, int pix_Y){ this->m_left_pix_X= pix_X; this->m_left_pix_Y= pix_Y;};
	void DrawTrigger( wxDC &dc);
	void DrawCursor( wxDC &dc);
	void DrawSamples( wxDC &dc);
	void DrawPosition( wxDC &dc);

	bool ToggleRecordStatus( const wxString &path, const wxString &file_name, const wxString &timestamp, bool start);

	// Pure virtual 
	virtual bool ReadSamples( bool do_record)= 0;
	virtual bool WriteDACOffset( void)= 0;
	virtual bool WriteChannelTrigger( void)= 0;

protected:
	bool m_is_recording;
	wxFile *m_p_record_file;

	wxMutex	*m_p_data_mutex;
#ifdef SAMPLE_LOCK
	wxMutex m_samples_mutex;
#endif
	cvt_board_data* m_p_data;
	int m_ch_index;
	AppSettings *m_p_app_settings;
	UINT16 *m_p_sample_buffer;
	UINT32 m_sample_buffer_size, m_sample_buffer_count, m_record_sample_counter;
	UINT16 *m_p_tmp_sample_buffer;
	UINT32 m_tmp_sample_buffer_size, m_tmp_sample_buffer_count, m_tmp_record_sample_counter;
	virtual bool RecordSamples( void);
	wxMutex m_record_mutex;
};


#endif	// _GENERIC_BOARD_CHANNEL_H_

