////////////////////////////////////////////////////////////////////////////////////////////////
/*! \file    cvt_V1495.c
*   \brief   V1495 VME board implementation
*   \author  NDA
*   \version 1.0
*   \date    03/2006
*            
*            Provides methods, properties and defines to handle with V1495 VME boards
*/
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
// File includes
////////////////////////////////////////////////////////////////////////////////////////////////
#include <memory.h>
#include <string.h>
#include "cvt_V1495.h"

////////////////////////////////////////////////////////////////////////////////////////////////
// File local defines
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
// Static variables declaration
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \var     static const cvt_reg_table CVT_V1495_REG_TABLE[]
*   \brief   The board registers table
*            
*            Provides an entry for each declared register: keep synched with \ref CVT_V1495_REG_INDEX
*/
////////////////////////////////////////////////////////////////////////////////////////////////
static const cvt_reg_table CVT_V1495_REG_TABLE[]=
{
	{ CVT_V1495_CONTROL_ADD,		CVT_V1495_CONTROL_AM,		CVT_V1495_CONTROL_DATA_SIZE},				/*!< \brief Control register */
	{ CVT_V1495_PORTA_EN_LOW_ADD,	CVT_V1495_PORTA_EN_LOW_AM,	CVT_V1495_PORTA_EN_LOW_DATA_SIZE},		/*!< \brief Port A enable low word register */
	{ CVT_V1495_PORTA_EN_HIGH_ADD,	CVT_V1495_PORTA_EN_HIGH_AM,	CVT_V1495_PORTA_EN_HIGH_DATA_SIZE},		/*!< \brief Port A enable high word register */
	{ CVT_V1495_PORTB_EN_LOW_ADD,	CVT_V1495_PORTB_EN_LOW_AM,	CVT_V1495_PORTB_EN_LOW_DATA_SIZE},		/*!< \brief Port B enable low word register */
	{ CVT_V1495_PORTB_EN_HIGH_ADD,	CVT_V1495_PORTB_EN_HIGH_AM,	CVT_V1495_PORTB_EN_HIGH_DATA_SIZE},		/*!< \brief Port B enable high word register */
	{ CVT_V1495_PORTD_EN_LOW_ADD,	CVT_V1495_PORTD_EN_LOW_AM,	CVT_V1495_PORTD_EN_LOW_DATA_SIZE},		/*!< \brief Port D enable low word register */
	{ CVT_V1495_PORTD_EN_HIGH_ADD,	CVT_V1495_PORTD_EN_HIGH_AM,	CVT_V1495_PORTD_EN_HIGH_DATA_SIZE},		/*!< \brief Port D enable high word register */

};
////////////////////////////////////////////////////////////////////////////////////////////////
// Static methods declaration
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
// Global visible variables declaration
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
//
//     B O A R D S   H A N D L I N G
//
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1495_open( cvt_V1495_data* p_data, UINT16 base_address, long vme_handle)
{
	memset( p_data, 0, sizeof( cvt_V1495_data));
	// basic data initialization
	if( !cvt_board_open( &p_data->m_common_data, base_address, vme_handle, CVT_V1495_REG_TABLE))
		return FALSE;
	// board specific data initialization

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1495_close( cvt_V1495_data* p_data)
{
	if( !cvt_board_close( &p_data->m_common_data))
		return FALSE;
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
//
//     L E V E L   0   A P I s
//
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
//
//     L E V E L   1   A P I s
//
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1495_set_control( cvt_V1495_data* p_data, UINT16 value)
{
	UINT16 reg_value= value;
	// write register
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V1495_CONTROL_INDEX, &reg_value))
	{
		TRACE( "V1495 CONTROL write failed !\n");
		return FALSE;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1495_set_port_A_enable( cvt_V1495_data* p_data, UINT32 value)
{
	UINT16 reg_value_l= (UINT16)( value& 0xffff);
	UINT16 reg_value_h= (UINT16)( ((UINT32)value>>16)& 0xffff);

	// write register
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V1495_PORTA_EN_LOW_INDEX, &reg_value_l))
	{
		TRACE( "V1495 PORT A enable write failed !\n");
		return FALSE;
	}
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V1495_PORTA_EN_HIGH_INDEX, &reg_value_h))
	{
		TRACE( "V1495 PORT A enable write failed !\n");
		return FALSE;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1495_set_port_B_enable( cvt_V1495_data* p_data, UINT32 value)
{
	UINT16 reg_value_l= (UINT16)( value& 0xffff);
	UINT16 reg_value_h= (UINT16)( ((UINT32)value>>16)& 0xffff);
	// write register
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V1495_PORTB_EN_LOW_INDEX, &reg_value_l))
	{
		TRACE( "V1495 PORT B enable write failed !\n");
		return FALSE;
	}
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V1495_PORTB_EN_HIGH_INDEX, &reg_value_h))
	{
		TRACE( "V1495 PORT B enable write failed !\n");
		return FALSE;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1495_set_port_D_enable( cvt_V1495_data* p_data, UINT32 value)
{
	UINT16 reg_value_l= (UINT16)( value& 0xffff);
	UINT16 reg_value_h= (UINT16)( ((UINT32)value>>16)& 0xffff);
	// write register
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V1495_PORTD_EN_LOW_INDEX, &reg_value_l))
	{
		TRACE( "V1495 PORT D enable write failed !\n");
		return FALSE;
	}
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V1495_PORTD_EN_HIGH_INDEX, &reg_value_h))
	{
		TRACE( "V1495 PORT D enable write failed !\n");
		return FALSE;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
//
//     L E V E L   2   A P I s
//
////////////////////////////////////////////////////////////////////////////////////////////////

