////////////////////////////////////////////////////////////////////////////////////////////////
/*! \file    cvt_V1724.h
*   \brief   V1724 VME board definitions
*   \author  NDA
*   \version 1.0
*   \date    10/2006
*            
*            Provides methods, properties and defines to handle with V1724 VME boards
*/
////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef __CVT_V1724_DEF_H
#define __CVT_V1724_DEF_H

////////////////////////////////////////////////////////////////////////////////////////////////
// File includes
////////////////////////////////////////////////////////////////////////////////////////////////
#include "cvt_common_defs.h"
#include "cvt_board_commons.h"
////////////////////////////////////////////////////////////////////////////////////////////////
// Global defines
////////////////////////////////////////////////////////////////////////////////////////////////

#define CVT_V1724_THRESHOLD_NUM			8			/*!< \brief Number of threshold registers */

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \def     CVT_V1724_USE_DATA_QUEUE
*   \brief   Enable/ disable V1724 builtin data queue
*            
*            Define this to enable V1724 builtin data queue; comment out to disable this feature
*/
////////////////////////////////////////////////////////////////////////////////////////////////
#define CVT_V1724_USE_DATA_QUEUE	1

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \struct  cvt_V1724_data
*   \brief   V1724 board data structure
*            
*            This structure stores any board data, common and specific ones. 
*            It must be passed to each V1724's API 
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct
{
	cvt_board_data	m_common_data;				/*!< Boards common data structure: always let at offset 0 */
	//
	// Board specific data

	UINT32 *m_tmp_sample_buffer;				/*!< Builtin temporary samples' buffer */
	UINT32 m_tmp_sample_buffer_size;			/*!< Builtin temporary samples' buffer size (UINT32 words)*/
} cvt_V1724_data;

////////////////////////////////////////////////////////////////////////////////////////////////
// Registers address
////////////////////////////////////////////////////////////////////////////////////////////////
//
// Channel broadcast registers
#define CVT_V1724_BROAD_CH_CTRL_ADD					0x8000				/*!< \brief Control channel broadcast register relative address */
#define CVT_V1724_BROAD_CH_SET_CTRL_ADD				0x8004				/*!< \brief Set control channel broadcast register relative address */
#define CVT_V1724_BROAD_CH_CLEAR_CTRL_ADD			0x8008				/*!< \brief Clear control channel broadcast register relative address */
#define CVT_V1724_BROAD_CH_BLKSIZE_ADD				0x800C				/*!< \brief Block size channel broadcast register relative address */
#define CVT_V1724_BROAD_CH_BLK_REM_NUM_ADD			0x8010				/*!< \brief Number of Blocks to Remove channel broadcast register relative address */
#define CVT_V1724_BROAD_CH_READ_CONF_ADD			0x8014				/*!< \brief Parameters to Read Block channel broadcast register relative address */
//
// VME registers
#define CVT_V1724_CONTROL_ADD						0x8104				/*!< \brief Control register relative address */
#define CVT_V1724_STATUS_ADD						0x8106				/*!< \brief Status register relative address */
#define CVT_V1724_INT_LEVEL_ADD						0x8108				/*!< \brief Interrupt level register relative address */
#define CVT_V1724_INT_VECTOR_ADD					0x810A				/*!< \brief Interrupt vector register relative address */
#define CVT_V1724_GEO_ADDRESS_ADD					0x810C				/*!< \brief Geo Address register relative address */
#define CVT_V1724_MCST_CBLT_ADDRESS_ADD				0x810E				/*!< \brief MCST/CBLT Address register relative address */
#define CVT_V1724_MCST_CBLT_CTRL_ADD				0x8110				/*!< \brief MCST/CBLT control register relative address */
#define CVT_V1724_SW_RESET_ADD						0x8112				/*!< \brief Software reset register relative address */
#define CVT_V1724_SW_CLEAR_ADD						0x8114				/*!< \brief Software clear register relative address */
#define CVT_V1724_SW_TRIGGER_ADD					0x8116				/*!< \brief Software trigger register relative address */
#define CVT_V1724_TRIGGER_ENABLE_ADD				0x811C				/*!< \brief Trigger enable register relative address */
#define CVT_V1724_BLT_EVENT_NUM_ADD					0x811E				/*!< \brief BLT event number register relative address */
#define CVT_V1724_FW_REV_ADD						0x8120				/*!< \brief Firmware Revision register relative address */
#define CVT_V1724_TEST_REG_ADD						0x8124				/*!< \brief Test register relative address */
#define CVT_V1724_FLASH_EN_ADD						0x812C				/*!< \brief Flash enable relative address */
#define CVT_V1724_FLASH_ADD							0x812E				/*!< \brief Flash relative address */
#define CVT_V1724_DUMMY16_ADD						0x8130				/*!< \brief Dummy 16 register relative address */
#define CVT_V1724_DUMMY32_ADD						0x8132				/*!< \brief Dummy 32 register relative address */
#define CVT_V1724_POST_TRIG_ADD						0x8136				/*!< \brief Post trigger register relative address */
#define CVT_V1724_FRONT_PANEL_IO_ADD				0x813C				/*!< \brief Front panel IO register relative address */
#define CVT_V1724_FRONT_PANEL_IO_CTRL_ADD			0x813E				/*!< \brief Front panel IO Control register relative address */
//
// CH 0
#define CVT_V1724_CH0_THRESHOLD_ADD					0x1080				/*!< \brief CH 0 Threshold register relative address */
#define CVT_V1724_CH0_THR_SAMPLE_ADD				0x1084				/*!< \brief CH 0 Over/Under Threshold Samples register relative address */
#define CVT_V1724_CH0_STATUS_ADD					0x1088				/*!< \brief CH 0 status register relative address */
#define CVT_V1724_CH0_FW_REV_ADD					0x108C				/*!< \brief CH 0 firmware revision register relative address */
#define CVT_V1724_CH0_READ_BLK_TRIG_ADD				0x1090				/*!< \brief CH 0 Read Block Trigger register relative address */
#define CVT_V1724_CH0_BLK_WRI_NUM_ADD				0x1094				/*!< \brief CH 0 Number of Blocks Written register relative address */
#define CVT_V1724_CH0_DAC_CONF_ADD					0x1098				/*!< \brief CH 0 DAC Data Configuration register relative address */
#define CVT_V1724_CH0_CONF_ADD						0x109C				/*!< \brief CH 0 Configuration register relative address */
#define CVT_V1724_CH0_DEBUG_ADD						0x10A0				/*!< \brief CH 0 Debug register relative address */
//
// CH 1
#define CVT_V1724_CH1_THRESHOLD_ADD					0x1180				/*!< \brief CH 1 Threshold register relative address */
#define CVT_V1724_CH1_THR_SAMPLE_ADD				0x1184				/*!< \brief CH 1 Over/Under Threshold Samples register relative address */
#define CVT_V1724_CH1_STATUS_ADD					0x1188				/*!< \brief CH 1 status register relative address */
#define CVT_V1724_CH1_FW_REV_ADD					0x118C				/*!< \brief CH 1 firmware revision register relative address */
#define CVT_V1724_CH1_READ_BLK_TRIG_ADD				0x1190				/*!< \brief CH 1 Read Block Trigger register relative address */
#define CVT_V1724_CH1_BLK_WRI_NUM_ADD				0x1194				/*!< \brief CH 1 Number of Blocks Written register relative address */
#define CVT_V1724_CH1_DAC_CONF_ADD					0x1198				/*!< \brief CH 1 DAC Data Configuration register relative address */
#define CVT_V1724_CH1_CONF_ADD						0x119C				/*!< \brief CH 1 Configuration register relative address */
#define CVT_V1724_CH1_DEBUG_ADD						0x11A0				/*!< \brief CH 1 Debug register relative address */
//
// CH 2
#define CVT_V1724_CH2_THRESHOLD_ADD					0x1280				/*!< \brief CH 2 Threshold register relative address */
#define CVT_V1724_CH2_THR_SAMPLE_ADD				0x1284				/*!< \brief CH 2 Over/Under Threshold Samples register relative address */
#define CVT_V1724_CH2_STATUS_ADD					0x1288				/*!< \brief CH 2 status register relative address */
#define CVT_V1724_CH2_FW_REV_ADD					0x128C				/*!< \brief CH 2 firmware revision register relative address */
#define CVT_V1724_CH2_READ_BLK_TRIG_ADD				0x1290				/*!< \brief CH 2 Read Block Trigger register relative address */
#define CVT_V1724_CH2_BLK_WRI_NUM_ADD				0x1294				/*!< \brief CH 2 Number of Blocks Written register relative address */
#define CVT_V1724_CH2_DAC_CONF_ADD					0x1298				/*!< \brief CH 2 DAC Data Configuration register relative address */
#define CVT_V1724_CH2_CONF_ADD						0x129C				/*!< \brief CH 2 Configuration register relative address */
#define CVT_V1724_CH2_DEBUG_ADD						0x12A0				/*!< \brief CH 2 Debug register relative address */
//
// CH 3
#define CVT_V1724_CH3_THRESHOLD_ADD					0x1380				/*!< \brief CH 3 Threshold register relative address */
#define CVT_V1724_CH3_THR_SAMPLE_ADD				0x1384				/*!< \brief CH 3 Over/Under Threshold Samples register relative address */
#define CVT_V1724_CH3_STATUS_ADD					0x1388				/*!< \brief CH 3 status register relative address */
#define CVT_V1724_CH3_FW_REV_ADD					0x138C				/*!< \brief CH 3 firmware revision register relative address */
#define CVT_V1724_CH3_READ_BLK_TRIG_ADD				0x1390				/*!< \brief CH 3 Read Block Trigger register relative address */
#define CVT_V1724_CH3_BLK_WRI_NUM_ADD				0x1394				/*!< \brief CH 3 Number of Blocks Written register relative address */
#define CVT_V1724_CH3_DAC_CONF_ADD					0x1398				/*!< \brief CH 3 DAC Data Configuration register relative address */
#define CVT_V1724_CH3_CONF_ADD						0x139C				/*!< \brief CH 3 Configuration register relative address */
#define CVT_V1724_CH3_DEBUG_ADD						0x13A0				/*!< \brief CH 3 Debug register relative address */
//
// CH 4
#define CVT_V1724_CH4_THRESHOLD_ADD					0x1480				/*!< \brief CH 4 Threshold register relative address */
#define CVT_V1724_CH4_THR_SAMPLE_ADD				0x1484				/*!< \brief CH 4 Over/Under Threshold Samples register relative address */
#define CVT_V1724_CH4_STATUS_ADD					0x1488				/*!< \brief CH 4 status register relative address */
#define CVT_V1724_CH4_FW_REV_ADD					0x148C				/*!< \brief CH 4 firmware revision register relative address */
#define CVT_V1724_CH4_READ_BLK_TRIG_ADD				0x1490				/*!< \brief CH 4 Read Block Trigger register relative address */
#define CVT_V1724_CH4_BLK_WRI_NUM_ADD				0x1494				/*!< \brief CH 4 Number of Blocks Written register relative address */
#define CVT_V1724_CH4_DAC_CONF_ADD					0x1498				/*!< \brief CH 4 DAC Data Configuration register relative address */
#define CVT_V1724_CH4_CONF_ADD						0x149C				/*!< \brief CH 4 Configuration register relative address */
#define CVT_V1724_CH4_DEBUG_ADD						0x14A0				/*!< \brief CH 4 Debug register relative address */
//
// CH 5
#define CVT_V1724_CH5_THRESHOLD_ADD					0x1580				/*!< \brief CH 5 Threshold register relative address */
#define CVT_V1724_CH5_THR_SAMPLE_ADD				0x1584				/*!< \brief CH 5 Over/Under Threshold Samples register relative address */
#define CVT_V1724_CH5_STATUS_ADD					0x1588				/*!< \brief CH 5 status register relative address */
#define CVT_V1724_CH5_FW_REV_ADD					0x158C				/*!< \brief CH 5 firmware revision register relative address */
#define CVT_V1724_CH5_READ_BLK_TRIG_ADD				0x1590				/*!< \brief CH 5 Read Block Trigger register relative address */
#define CVT_V1724_CH5_BLK_WRI_NUM_ADD				0x1594				/*!< \brief CH 5 Number of Blocks Written register relative address */
#define CVT_V1724_CH5_DAC_CONF_ADD					0x1598				/*!< \brief CH 5 DAC Data Configuration register relative address */
#define CVT_V1724_CH5_CONF_ADD						0x159C				/*!< \brief CH 5 Configuration register relative address */
#define CVT_V1724_CH5_DEBUG_ADD						0x15A0				/*!< \brief CH 5 Debug register relative address */
//
// CH 6
#define CVT_V1724_CH6_THRESHOLD_ADD					0x1680				/*!< \brief CH 6 Threshold register relative address */
#define CVT_V1724_CH6_THR_SAMPLE_ADD				0x1684				/*!< \brief CH 6 Over/Under Threshold Samples register relative address */
#define CVT_V1724_CH6_STATUS_ADD					0x1688				/*!< \brief CH 6 status register relative address */
#define CVT_V1724_CH6_FW_REV_ADD					0x168C				/*!< \brief CH 6 firmware revision register relative address */
#define CVT_V1724_CH6_READ_BLK_TRIG_ADD				0x1690				/*!< \brief CH 6 Read Block Trigger register relative address */
#define CVT_V1724_CH6_BLK_WRI_NUM_ADD				0x1694				/*!< \brief CH 6 Number of Blocks Written register relative address */
#define CVT_V1724_CH6_DAC_CONF_ADD					0x1698				/*!< \brief CH 6 DAC Data Configuration register relative address */
#define CVT_V1724_CH6_CONF_ADD						0x169C				/*!< \brief CH 6 Configuration register relative address */
#define CVT_V1724_CH6_DEBUG_ADD						0x16A0				/*!< \brief CH 6 Debug register relative address */
//
// CH 7
#define CVT_V1724_CH7_THRESHOLD_ADD					0x1780				/*!< \brief CH 7 Threshold register relative address */
#define CVT_V1724_CH7_THR_SAMPLE_ADD				0x1784				/*!< \brief CH 7 Over/Under Threshold Samples register relative address */
#define CVT_V1724_CH7_STATUS_ADD					0x1788				/*!< \brief CH 7 status register relative address */
#define CVT_V1724_CH7_FW_REV_ADD					0x178C				/*!< \brief CH 7 firmware revision register relative address */
#define CVT_V1724_CH7_READ_BLK_TRIG_ADD				0x1790				/*!< \brief CH 7 Read Block Trigger register relative address */
#define CVT_V1724_CH7_BLK_WRI_NUM_ADD				0x1794				/*!< \brief CH 7 Number of Blocks Written register relative address */
#define CVT_V1724_CH7_DAC_CONF_ADD					0x1798				/*!< \brief CH 7 DAC Data Configuration register relative address */
#define CVT_V1724_CH7_CONF_ADD						0x179C				/*!< \brief CH 7 Configuration register relative address */
#define CVT_V1724_CH7_DEBUG_ADD						0x17A0				/*!< \brief CH 7 Debug register relative address */

////////////////////////////////////////////////////////////////////////////////////////////////
// Registers data size
////////////////////////////////////////////////////////////////////////////////////////////////
//
// Channel broadcast registers
#define CVT_V1724_BROAD_CH_CTRL_DATA_SIZE			cvD32				/*!< \brief Control channel broadcast register data size */
#define CVT_V1724_BROAD_CH_SET_CTRL_DATA_SIZE		cvD32				/*!< \brief Set control channel broadcast register data size */
#define CVT_V1724_BROAD_CH_CLEAR_CTRL_DATA_SIZE		cvD32				/*!< \brief Clear control channel broadcast register data size */
#define CVT_V1724_BROAD_CH_BLKSIZE_DATA_SIZE		cvD32				/*!< \brief Block size channel broadcast register data size */
#define CVT_V1724_BROAD_CH_BLK_REM_NUM_DATA_SIZE	cvD32				/*!< \brief Number of Blocks to Remove channel broadcast register data size */
#define CVT_V1724_BROAD_CH_READ_CONF_DATA_SIZE		cvD32				/*!< \brief Parameters to Read Block channel broadcast register data size */
//
// VME registers
#define CVT_V1724_CONTROL_DATA_SIZE					cvD16				/*!< \brief Control register data size */
#define CVT_V1724_STATUS_DATA_SIZE					cvD16				/*!< \brief Status register data size */
#define CVT_V1724_INT_LEVEL_DATA_SIZE				cvD16				/*!< \brief Interrupt level register data size */
#define CVT_V1724_INT_VECTOR_DATA_SIZE				cvD16				/*!< \brief Interrupt vector register data size */
#define CVT_V1724_GEO_ADDRESS_DATA_SIZE				cvD16				/*!< \brief Geo Address register data size */
#define CVT_V1724_MCST_CBLT_ADDRESS_DATA_SIZE		cvD16				/*!< \brief MCST/CBLT Address register data size */
#define CVT_V1724_MCST_CBLT_CTRL_DATA_SIZE			cvD16				/*!< \brief MCST/CBLT control register data size */
#define CVT_V1724_SW_RESET_DATA_SIZE				cvD16				/*!< \brief Software reset register data size */
#define CVT_V1724_SW_CLEAR_DATA_SIZE				cvD16				/*!< \brief Software clear register data size */
#define CVT_V1724_SW_TRIGGER_DATA_SIZE				cvD16				/*!< \brief Software trigger register data size */
#define CVT_V1724_TRIGGER_ENABLE_DATA_SIZE			cvD16				/*!< \brief Trigger enable register data size */
#define CVT_V1724_BLT_EVENT_NUM_DATA_SIZE			cvD16				/*!< \brief BLT event number register data size */
#define CVT_V1724_FW_REV_DATA_SIZE					cvD32				/*!< \brief Firmware Revision register data size */
#define CVT_V1724_TEST_REG_DATA_SIZE				cvD32				/*!< \brief Test register data size */
#define CVT_V1724_FLASH_EN_DATA_SIZE				cvD16				/*!< \brief Flash enable data size */
#define CVT_V1724_FLASH_DATA_SIZE					cvD16				/*!< \brief Flash data size */
#define CVT_V1724_DUMMY16_DATA_SIZE					cvD16				/*!< \brief Dummy 16 register data size */
#define CVT_V1724_DUMMY32_DATA_SIZE					cvD32				/*!< \brief Dummy 32 register data size */
#define CVT_V1724_POST_TRIG_DATA_SIZE				cvD32				/*!< \brief Post trigger register data size */
#define CVT_V1724_FRONT_PANEL_IO_DATA_SIZE			cvD16				/*!< \brief Front panel IO register data size */
#define CVT_V1724_FRONT_PANEL_IO_CTRL_DATA_SIZE		cvD16				/*!< \brief Front panel IO Control register data size */

//
// CH 0
#define CVT_V1724_CH0_THRESHOLD_DATA_SIZE			cvD32				/*!< \brief CH 0 Threshold register relative data size */
#define CVT_V1724_CH0_THR_SAMPLE_DATA_SIZE			cvD32				/*!< \brief CH 0 Over/Under Threshold Samples register relative data size */
#define CVT_V1724_CH0_STATUS_DATA_SIZE				cvD32				/*!< \brief CH 0 status register relative data size */
#define CVT_V1724_CH0_FW_REV_DATA_SIZE				cvD32				/*!< \brief CH 0 firmware revision register relative data size */
#define CVT_V1724_CH0_READ_BLK_TRIG_DATA_SIZE		cvD32				/*!< \brief CH 0 Read Block Trigger register relative data size */
#define CVT_V1724_CH0_BLK_WRI_NUM_DATA_SIZE			cvD32				/*!< \brief CH 0 Number of Blocks Written register relative data size */
#define CVT_V1724_CH0_DAC_CONF_DATA_SIZE			cvD32				/*!< \brief CH 0 DAC Data Configuration register relative data size */
#define CVT_V1724_CH0_CONF_DATA_SIZE				cvD32				/*!< \brief CH 0 Configuration register relative data size */
#define CVT_V1724_CH0_DEBUG_DATA_SIZE				cvD32				/*!< \brief CH 0 Debug register relative data size */
//
// CH 1
#define CVT_V1724_CH1_THRESHOLD_DATA_SIZE			cvD32				/*!< \brief CH 1 Threshold register relative data size */
#define CVT_V1724_CH1_THR_SAMPLE_DATA_SIZE			cvD32				/*!< \brief CH 1 Over/Under Threshold Samples register relative data size */
#define CVT_V1724_CH1_STATUS_DATA_SIZE				cvD32				/*!< \brief CH 1 status register relative data size */
#define CVT_V1724_CH1_FW_REV_DATA_SIZE				cvD32				/*!< \brief CH 1 firmware revision register relative data size */
#define CVT_V1724_CH1_READ_BLK_TRIG_DATA_SIZE		cvD32				/*!< \brief CH 1 Read Block Trigger register relative data size */
#define CVT_V1724_CH1_BLK_WRI_NUM_DATA_SIZE			cvD32				/*!< \brief CH 1 Number of Blocks Written register relative data size */
#define CVT_V1724_CH1_DAC_CONF_DATA_SIZE			cvD32				/*!< \brief CH 1 DAC Data Configuration register relative data size */
#define CVT_V1724_CH1_CONF_DATA_SIZE				cvD32				/*!< \brief CH 1 Configuration register relative data size */
#define CVT_V1724_CH1_DEBUG_DATA_SIZE				cvD32				/*!< \brief CH 1 Debug register relative data size */
//
// CH 2
#define CVT_V1724_CH2_THRESHOLD_DATA_SIZE			cvD32				/*!< \brief CH 2 Threshold register relative data size */
#define CVT_V1724_CH2_THR_SAMPLE_DATA_SIZE			cvD32				/*!< \brief CH 2 Over/Under Threshold Samples register relative data size */
#define CVT_V1724_CH2_STATUS_DATA_SIZE				cvD32				/*!< \brief CH 2 status register relative data size */
#define CVT_V1724_CH2_FW_REV_DATA_SIZE				cvD32				/*!< \brief CH 2 firmware revision register relative data size */
#define CVT_V1724_CH2_READ_BLK_TRIG_DATA_SIZE		cvD32				/*!< \brief CH 2 Read Block Trigger register relative data size */
#define CVT_V1724_CH2_BLK_WRI_NUM_DATA_SIZE			cvD32				/*!< \brief CH 2 Number of Blocks Written register relative data size */
#define CVT_V1724_CH2_DAC_CONF_DATA_SIZE			cvD32				/*!< \brief CH 2 DAC Data Configuration register relative data size */
#define CVT_V1724_CH2_CONF_DATA_SIZE				cvD32				/*!< \brief CH 2 Configuration register relative data size */
#define CVT_V1724_CH2_DEBUG_DATA_SIZE				cvD32				/*!< \brief CH 2 Debug register relative data size */
//
// CH 3
#define CVT_V1724_CH3_THRESHOLD_DATA_SIZE			cvD32				/*!< \brief CH 3 Threshold register relative data size */
#define CVT_V1724_CH3_THR_SAMPLE_DATA_SIZE			cvD32				/*!< \brief CH 3 Over/Under Threshold Samples register relative data size */
#define CVT_V1724_CH3_STATUS_DATA_SIZE				cvD32				/*!< \brief CH 3 status register relative data size */
#define CVT_V1724_CH3_FW_REV_DATA_SIZE				cvD32				/*!< \brief CH 3 firmware revision register relative data size */
#define CVT_V1724_CH3_READ_BLK_TRIG_DATA_SIZE		cvD32				/*!< \brief CH 3 Read Block Trigger register relative data size */
#define CVT_V1724_CH3_BLK_WRI_NUM_DATA_SIZE			cvD32				/*!< \brief CH 3 Number of Blocks Written register relative data size */
#define CVT_V1724_CH3_DAC_CONF_DATA_SIZE			cvD32				/*!< \brief CH 3 DAC Data Configuration register relative data size */
#define CVT_V1724_CH3_CONF_DATA_SIZE				cvD32				/*!< \brief CH 3 Configuration register relative data size */
#define CVT_V1724_CH3_DEBUG_DATA_SIZE				cvD32				/*!< \brief CH 3 Debug register relative data size */
//
// CH 4
#define CVT_V1724_CH4_THRESHOLD_DATA_SIZE			cvD32				/*!< \brief CH 4 Threshold register relative data size */
#define CVT_V1724_CH4_THR_SAMPLE_DATA_SIZE			cvD32				/*!< \brief CH 4 Over/Under Threshold Samples register relative data size */
#define CVT_V1724_CH4_STATUS_DATA_SIZE				cvD32				/*!< \brief CH 4 status register relative data size */
#define CVT_V1724_CH4_FW_REV_DATA_SIZE				cvD32				/*!< \brief CH 4 firmware revision register relative data size */
#define CVT_V1724_CH4_READ_BLK_TRIG_DATA_SIZE		cvD32				/*!< \brief CH 4 Read Block Trigger register relative data size */
#define CVT_V1724_CH4_BLK_WRI_NUM_DATA_SIZE			cvD32				/*!< \brief CH 4 Number of Blocks Written register relative data size */
#define CVT_V1724_CH4_DAC_CONF_DATA_SIZE			cvD32				/*!< \brief CH 4 DAC Data Configuration register relative data size */
#define CVT_V1724_CH4_CONF_DATA_SIZE				cvD32				/*!< \brief CH 4 Configuration register relative data size */
#define CVT_V1724_CH4_DEBUG_DATA_SIZE				cvD32				/*!< \brief CH 4 Debug register relative data size */
//
// CH 5
#define CVT_V1724_CH5_THRESHOLD_DATA_SIZE			cvD32				/*!< \brief CH 5 Threshold register relative data size */
#define CVT_V1724_CH5_THR_SAMPLE_DATA_SIZE			cvD32				/*!< \brief CH 5 Over/Under Threshold Samples register relative data size */
#define CVT_V1724_CH5_STATUS_DATA_SIZE				cvD32				/*!< \brief CH 5 status register relative data size */
#define CVT_V1724_CH5_FW_REV_DATA_SIZE				cvD32				/*!< \brief CH 5 firmware revision register relative data size */
#define CVT_V1724_CH5_READ_BLK_TRIG_DATA_SIZE		cvD32				/*!< \brief CH 5 Read Block Trigger register relative data size */
#define CVT_V1724_CH5_BLK_WRI_NUM_DATA_SIZE			cvD32				/*!< \brief CH 5 Number of Blocks Written register relative data size */
#define CVT_V1724_CH5_DAC_CONF_DATA_SIZE			cvD32				/*!< \brief CH 5 DAC Data Configuration register relative data size */
#define CVT_V1724_CH5_CONF_DATA_SIZE				cvD32				/*!< \brief CH 5 Configuration register relative data size */
#define CVT_V1724_CH5_DEBUG_DATA_SIZE				cvD32				/*!< \brief CH 5 Debug register relative data size */
//
// CH 6
#define CVT_V1724_CH6_THRESHOLD_DATA_SIZE			cvD32				/*!< \brief CH 6 Threshold register relative data size */
#define CVT_V1724_CH6_THR_SAMPLE_DATA_SIZE			cvD32				/*!< \brief CH 6 Over/Under Threshold Samples register relative data size */
#define CVT_V1724_CH6_STATUS_DATA_SIZE				cvD32				/*!< \brief CH 6 status register relative data size */
#define CVT_V1724_CH6_FW_REV_DATA_SIZE				cvD32				/*!< \brief CH 6 firmware revision register relative data size */
#define CVT_V1724_CH6_READ_BLK_TRIG_DATA_SIZE		cvD32				/*!< \brief CH 6 Read Block Trigger register relative data size */
#define CVT_V1724_CH6_BLK_WRI_NUM_DATA_SIZE			cvD32				/*!< \brief CH 6 Number of Blocks Written register relative data size */
#define CVT_V1724_CH6_DAC_CONF_DATA_SIZE			cvD32				/*!< \brief CH 6 DAC Data Configuration register relative data size */
#define CVT_V1724_CH6_CONF_DATA_SIZE				cvD32				/*!< \brief CH 6 Configuration register relative data size */
#define CVT_V1724_CH6_DEBUG_DATA_SIZE				cvD32				/*!< \brief CH 6 Debug register relative data size */
//
// CH 7
#define CVT_V1724_CH7_THRESHOLD_DATA_SIZE			cvD32				/*!< \brief CH 7 Threshold register relative data size */
#define CVT_V1724_CH7_THR_SAMPLE_DATA_SIZE			cvD32				/*!< \brief CH 7 Over/Under Threshold Samples register relative data size */
#define CVT_V1724_CH7_STATUS_DATA_SIZE				cvD32				/*!< \brief CH 7 status register relative data size */
#define CVT_V1724_CH7_FW_REV_DATA_SIZE				cvD32				/*!< \brief CH 7 firmware revision register relative data size */
#define CVT_V1724_CH7_READ_BLK_TRIG_DATA_SIZE		cvD32				/*!< \brief CH 7 Read Block Trigger register relative data size */
#define CVT_V1724_CH7_BLK_WRI_NUM_DATA_SIZE			cvD32				/*!< \brief CH 7 Number of Blocks Written register relative data size */
#define CVT_V1724_CH7_DAC_CONF_DATA_SIZE			cvD32				/*!< \brief CH 7 DAC Data Configuration register relative data size */
#define CVT_V1724_CH7_CONF_DATA_SIZE				cvD32				/*!< \brief CH 7 Configuration register relative data size */
#define CVT_V1724_CH7_DEBUG_DATA_SIZE				cvD32				/*!< \brief CH 7 Debug register relative data size */

////////////////////////////////////////////////////////////////////////////////////////////////
// Registers address modifiers
////////////////////////////////////////////////////////////////////////////////////////////////
//
// Channel broadcast registers
#define CVT_V1724_BROAD_CH_CTRL_AM			cvA32_S_DATA				/*!< \brief Control channel broadcast register address modifier */
#define CVT_V1724_BROAD_CH_SET_CTRL_AM		cvA32_S_DATA				/*!< \brief Set control channel broadcast register address modifier */
#define CVT_V1724_BROAD_CH_CLEAR_CTRL_AM	cvA32_S_DATA				/*!< \brief Clear control channel broadcast register address modifier */
#define CVT_V1724_BROAD_CH_BLKSIZE_AM		cvA32_S_DATA				/*!< \brief Block size channel broadcast register address modifier */
#define CVT_V1724_BROAD_CH_BLK_REM_NUM_AM	cvA32_S_DATA				/*!< \brief Number of Blocks to Remove channel broadcast register address modifier */
#define CVT_V1724_BROAD_CH_READ_CONF_AM		cvA32_S_DATA				/*!< \brief Parameters to Read Block channel broadcast register address modifier */
//
// VME registers
#define CVT_V1724_CONTROL_AM				cvA32_S_DATA				/*!< \brief Control register address modifier */
#define CVT_V1724_STATUS_AM					cvA32_S_DATA				/*!< \brief Status register address modifier */
#define CVT_V1724_INT_LEVEL_AM				cvA32_S_DATA				/*!< \brief Interrupt level register address modifier */
#define CVT_V1724_INT_VECTOR_AM				cvA32_S_DATA				/*!< \brief Interrupt vector register address modifier */
#define CVT_V1724_GEO_ADDRESS_AM			cvA32_S_DATA				/*!< \brief Geo Address register address modifier */
#define CVT_V1724_MCST_CBLT_ADDRESS_AM		cvA32_S_DATA				/*!< \brief MCST/CBLT Address register address modifier */
#define CVT_V1724_MCST_CBLT_CTRL_AM			cvA32_S_DATA				/*!< \brief MCST/CBLT control register address modifier */
#define CVT_V1724_SW_RESET_AM				cvA32_S_DATA				/*!< \brief Software reset register address modifier */
#define CVT_V1724_SW_CLEAR_AM				cvA32_S_DATA				/*!< \brief Software clear register address modifier */
#define CVT_V1724_SW_TRIGGER_AM				cvA32_S_DATA				/*!< \brief Software trigger register address modifier */
#define CVT_V1724_TRIGGER_ENABLE_AM			cvA32_S_DATA				/*!< \brief Trigger enable register address modifier */
#define CVT_V1724_BLT_EVENT_NUM_AM			cvA32_S_DATA				/*!< \brief BLT event number register address modifier */
#define CVT_V1724_FW_REV_AM					cvA32_S_DATA				/*!< \brief Firmware Revision register address modifier */
#define CVT_V1724_TEST_REG_AM				cvA32_S_DATA				/*!< \brief Test register address modifier */
#define CVT_V1724_FLASH_EN_AM				cvA32_S_DATA				/*!< \brief Flash enable address modifier */
#define CVT_V1724_FLASH_AM					cvA32_S_DATA				/*!< \brief Flash address modifier */
#define CVT_V1724_DUMMY16_AM				cvA32_S_DATA				/*!< \brief Dummy 16 register address modifier */
#define CVT_V1724_DUMMY32_AM				cvA32_S_DATA				/*!< \brief Dummy 32 register address modifier */
#define CVT_V1724_POST_TRIG_AM				cvA32_S_DATA				/*!< \brief Post trigger register address modifier */
#define CVT_V1724_FRONT_PANEL_IO_AM			cvA32_S_DATA				/*!< \brief Front panel IO register address modifier */
#define CVT_V1724_FRONT_PANEL_IO_CTRL_AM	cvA32_S_DATA				/*!< \brief Front panel IO Control register address modifier */

//
// CH 0
#define CVT_V1724_CH0_THRESHOLD_AM			cvA32_S_DATA				/*!< \brief CH 0 Threshold register relative address modifier */
#define CVT_V1724_CH0_THR_SAMPLE_AM			cvA32_S_DATA				/*!< \brief CH 0 Over/Under Threshold Samples register relative address modifier */
#define CVT_V1724_CH0_STATUS_AM				cvA32_S_DATA				/*!< \brief CH 0 status register relative address modifier */
#define CVT_V1724_CH0_FW_REV_AM				cvA32_S_DATA				/*!< \brief CH 0 firmware revision register relative address modifier */
#define CVT_V1724_CH0_READ_BLK_TRIG_AM		cvA32_S_BLT					/*!< \brief CH 0 Read Block Trigger register relative address modifier */
#define CVT_V1724_CH0_BLK_WRI_NUM_AM		cvA32_S_DATA				/*!< \brief CH 0 Number of Blocks Written register relative address modifier */
#define CVT_V1724_CH0_DAC_CONF_AM			cvA32_S_DATA				/*!< \brief CH 0 DAC Data Configuration register relative address modifier */
#define CVT_V1724_CH0_CONF_AM				cvA32_S_DATA				/*!< \brief CH 0 Configuration register relative address modifier */
#define CVT_V1724_CH0_DEBUG_AM				cvA32_S_DATA				/*!< \brief CH 0 Debug register relative address modifier */
//
// CH 1
#define CVT_V1724_CH1_THRESHOLD_AM			cvA32_S_DATA				/*!< \brief CH 1 Threshold register relative address modifier */
#define CVT_V1724_CH1_THR_SAMPLE_AM			cvA32_S_DATA				/*!< \brief CH 1 Over/Under Threshold Samples register relative address modifier */
#define CVT_V1724_CH1_STATUS_AM				cvA32_S_DATA				/*!< \brief CH 1 status register relative address modifier */
#define CVT_V1724_CH1_FW_REV_AM				cvA32_S_DATA				/*!< \brief CH 1 firmware revision register relative address modifier */
#define CVT_V1724_CH1_READ_BLK_TRIG_AM		cvA32_S_BLT					/*!< \brief CH 1 Read Block Trigger register relative address modifier */
#define CVT_V1724_CH1_BLK_WRI_NUM_AM		cvA32_S_DATA				/*!< \brief CH 1 Number of Blocks Written register relative address modifier */
#define CVT_V1724_CH1_DAC_CONF_AM			cvA32_S_DATA				/*!< \brief CH 1 DAC Data Configuration register relative address modifier */
#define CVT_V1724_CH1_CONF_AM				cvA32_S_DATA				/*!< \brief CH 1 Configuration register relative address modifier */
#define CVT_V1724_CH1_DEBUG_AM				cvA32_S_DATA				/*!< \brief CH 1 Debug register relative address modifier */
//
// CH 2
#define CVT_V1724_CH2_THRESHOLD_AM			cvA32_S_DATA				/*!< \brief CH 2 Threshold register relative address modifier */
#define CVT_V1724_CH2_THR_SAMPLE_AM			cvA32_S_DATA				/*!< \brief CH 2 Over/Under Threshold Samples register relative address modifier */
#define CVT_V1724_CH2_STATUS_AM				cvA32_S_DATA				/*!< \brief CH 2 status register relative address modifier */
#define CVT_V1724_CH2_FW_REV_AM				cvA32_S_DATA				/*!< \brief CH 2 firmware revision register relative address modifier */
#define CVT_V1724_CH2_READ_BLK_TRIG_AM		cvA32_S_BLT					/*!< \brief CH 2 Read Block Trigger register relative address modifier */
#define CVT_V1724_CH2_BLK_WRI_NUM_AM		cvA32_S_DATA				/*!< \brief CH 2 Number of Blocks Written register relative address modifier */
#define CVT_V1724_CH2_DAC_CONF_AM			cvA32_S_DATA				/*!< \brief CH 2 DAC Data Configuration register relative address modifier */
#define CVT_V1724_CH2_CONF_AM				cvA32_S_DATA				/*!< \brief CH 2 Configuration register relative address modifier */
#define CVT_V1724_CH2_DEBUG_AM				cvA32_S_DATA				/*!< \brief CH 2 Debug register relative address modifier */
//
// CH 3
#define CVT_V1724_CH3_THRESHOLD_AM			cvA32_S_DATA				/*!< \brief CH 3 Threshold register relative address modifier */
#define CVT_V1724_CH3_THR_SAMPLE_AM			cvA32_S_DATA				/*!< \brief CH 3 Over/Under Threshold Samples register relative address modifier */
#define CVT_V1724_CH3_STATUS_AM				cvA32_S_DATA				/*!< \brief CH 3 status register relative address modifier */
#define CVT_V1724_CH3_FW_REV_AM				cvA32_S_DATA				/*!< \brief CH 3 firmware revision register relative address modifier */
#define CVT_V1724_CH3_READ_BLK_TRIG_AM		cvA32_S_BLT					/*!< \brief CH 3 Read Block Trigger register relative address modifier */
#define CVT_V1724_CH3_BLK_WRI_NUM_AM		cvA32_S_DATA				/*!< \brief CH 3 Number of Blocks Written register relative address modifier */
#define CVT_V1724_CH3_DAC_CONF_AM			cvA32_S_DATA				/*!< \brief CH 3 DAC Data Configuration register relative address modifier */
#define CVT_V1724_CH3_CONF_AM				cvA32_S_DATA				/*!< \brief CH 3 Configuration register relative address modifier */
#define CVT_V1724_CH3_DEBUG_AM				cvA32_S_DATA				/*!< \brief CH 3 Debug register relative address modifier */
//
// CH 4
#define CVT_V1724_CH4_THRESHOLD_AM			cvA32_S_DATA				/*!< \brief CH 4 Threshold register relative address modifier */
#define CVT_V1724_CH4_THR_SAMPLE_AM			cvA32_S_DATA				/*!< \brief CH 4 Over/Under Threshold Samples register relative address modifier */
#define CVT_V1724_CH4_STATUS_AM				cvA32_S_DATA				/*!< \brief CH 4 status register relative address modifier */
#define CVT_V1724_CH4_FW_REV_AM				cvA32_S_DATA				/*!< \brief CH 4 firmware revision register relative address modifier */
#define CVT_V1724_CH4_READ_BLK_TRIG_AM		cvA32_S_BLT					/*!< \brief CH 4 Read Block Trigger register relative address modifier */
#define CVT_V1724_CH4_BLK_WRI_NUM_AM		cvA32_S_DATA				/*!< \brief CH 4 Number of Blocks Written register relative address modifier */
#define CVT_V1724_CH4_DAC_CONF_AM			cvA32_S_DATA				/*!< \brief CH 4 DAC Data Configuration register relative address modifier */
#define CVT_V1724_CH4_CONF_AM				cvA32_S_DATA				/*!< \brief CH 4 Configuration register relative address modifier */
#define CVT_V1724_CH4_DEBUG_AM				cvA32_S_DATA				/*!< \brief CH 4 Debug register relative address modifier */
//
// CH 5
#define CVT_V1724_CH5_THRESHOLD_AM			cvA32_S_DATA				/*!< \brief CH 5 Threshold register relative address modifier */
#define CVT_V1724_CH5_THR_SAMPLE_AM			cvA32_S_DATA				/*!< \brief CH 5 Over/Under Threshold Samples register relative address modifier */
#define CVT_V1724_CH5_STATUS_AM				cvA32_S_DATA				/*!< \brief CH 5 status register relative address modifier */
#define CVT_V1724_CH5_FW_REV_AM				cvA32_S_DATA				/*!< \brief CH 5 firmware revision register relative address modifier */
#define CVT_V1724_CH5_READ_BLK_TRIG_AM		cvA32_S_BLT					/*!< \brief CH 5 Read Block Trigger register relative address modifier */
#define CVT_V1724_CH5_BLK_WRI_NUM_AM		cvA32_S_DATA				/*!< \brief CH 5 Number of Blocks Written register relative address modifier */
#define CVT_V1724_CH5_DAC_CONF_AM			cvA32_S_DATA				/*!< \brief CH 5 DAC Data Configuration register relative address modifier */
#define CVT_V1724_CH5_CONF_AM				cvA32_S_DATA				/*!< \brief CH 5 Configuration register relative address modifier */
#define CVT_V1724_CH5_DEBUG_AM				cvA32_S_DATA				/*!< \brief CH 5 Debug register relative address modifier */
//
// CH 6
#define CVT_V1724_CH6_THRESHOLD_AM			cvA32_S_DATA				/*!< \brief CH 6 Threshold register relative address modifier */
#define CVT_V1724_CH6_THR_SAMPLE_AM			cvA32_S_DATA				/*!< \brief CH 6 Over/Under Threshold Samples register relative address modifier */
#define CVT_V1724_CH6_STATUS_AM				cvA32_S_DATA				/*!< \brief CH 6 status register relative address modifier */
#define CVT_V1724_CH6_FW_REV_AM				cvA32_S_DATA				/*!< \brief CH 6 firmware revision register relative address modifier */
#define CVT_V1724_CH6_READ_BLK_TRIG_AM		cvA32_S_BLT					/*!< \brief CH 6 Read Block Trigger register relative address modifier */
#define CVT_V1724_CH6_BLK_WRI_NUM_AM		cvA32_S_DATA				/*!< \brief CH 6 Number of Blocks Written register relative address modifier */
#define CVT_V1724_CH6_DAC_CONF_AM			cvA32_S_DATA				/*!< \brief CH 6 DAC Data Configuration register relative address modifier */
#define CVT_V1724_CH6_CONF_AM				cvA32_S_DATA				/*!< \brief CH 6 Configuration register relative address modifier */
#define CVT_V1724_CH6_DEBUG_AM				cvA32_S_DATA				/*!< \brief CH 6 Debug register relative address modifier */
//
// CH 7
#define CVT_V1724_CH7_THRESHOLD_AM			cvA32_S_DATA				/*!< \brief CH 7 Threshold register relative address modifier */
#define CVT_V1724_CH7_THR_SAMPLE_AM			cvA32_S_DATA				/*!< \brief CH 7 Over/Under Threshold Samples register relative address modifier */
#define CVT_V1724_CH7_STATUS_AM				cvA32_S_DATA				/*!< \brief CH 7 status register relative address modifier */
#define CVT_V1724_CH7_FW_REV_AM				cvA32_S_DATA				/*!< \brief CH 7 firmware revision register relative address modifier */
#define CVT_V1724_CH7_READ_BLK_TRIG_AM		cvA32_S_BLT					/*!< \brief CH 7 Read Block Trigger register relative address modifier */
#define CVT_V1724_CH7_BLK_WRI_NUM_AM		cvA32_S_DATA				/*!< \brief CH 7 Number of Blocks Written register relative address modifier */
#define CVT_V1724_CH7_DAC_CONF_AM			cvA32_S_DATA				/*!< \brief CH 7 DAC Data Configuration register relative address modifier */
#define CVT_V1724_CH7_CONF_AM				cvA32_S_DATA				/*!< \brief CH 7 Configuration register relative address modifier */
#define CVT_V1724_CH7_DEBUG_AM				cvA32_S_DATA				/*!< \brief CH 7 Debug register relative address modifier */

////////////////////////////////////////////////////////////////////////////////////////////////
// Registers indexes
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V792_REG_INDEX
*   \brief   The registers indexes
*            
*            Provides an entry for each register: This is the index into the CVT_V792_REG_TABLE board table
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
//
// Channel broadcast registers
	CVT_V1724_BROAD_CH_CTRL_INDEX,					/*!< \brief Control channel broadcast register index */
	CVT_V1724_BROAD_CH_SET_CTRL_INDEX,				/*!< \brief Set control channel broadcast register index */
	CVT_V1724_BROAD_CH_CLEAR_CTRL_INDEX,			/*!< \brief Clear control channel broadcast register index */
	CVT_V1724_BROAD_CH_BLKSIZE_INDEX,				/*!< \brief Block size channel broadcast register index */
	CVT_V1724_BROAD_CH_BLK_REM_NUM_INDEX,			/*!< \brief Number of Blocks to Remove channel broadcast register index */
	CVT_V1724_BROAD_CH_READ_CONF_INDEX,				/*!< \brief Parameters to Read Block channel broadcast register index */
//
// VME registers
	CVT_V1724_CONTROL_INDEX,						/*!< \brief Control register index */
	CVT_V1724_STATUS_INDEX,							/*!< \brief Status register index */
	CVT_V1724_INT_LEVEL_INDEX,						/*!< \brief Interrupt level register index */
	CVT_V1724_INT_VECTOR_INDEX,						/*!< \brief Interrupt vector register index */
	CVT_V1724_GEO_ADDRESS_INDEX,					/*!< \brief Geo Address register index */
	CVT_V1724_MCST_CBLT_ADDRESS_INDEX,				/*!< \brief MCST/CBLT Address register index */
	CVT_V1724_MCST_CBLT_CTRL_INDEX,					/*!< \brief MCST/CBLT control register index */
	CVT_V1724_SW_RESET_INDEX,						/*!< \brief Software reset register index */
	CVT_V1724_SW_CLEAR_INDEX,						/*!< \brief Software clear register index */
	CVT_V1724_SW_TRIGGER_INDEX,						/*!< \brief Software trigger register index */
	CVT_V1724_TRIGGER_ENABLE_INDEX,					/*!< \brief Trigger enable register index */
	CVT_V1724_BLT_EVENT_NUM_INDEX,					/*!< \brief BLT event number register index */
	CVT_V1724_FW_REV_INDEX,							/*!< \brief Firmware Revision register index */
	CVT_V1724_TEST_REG_INDEX,						/*!< \brief Test register index */
	CVT_V1724_FLASH_EN_INDEX,						/*!< \brief Flash enable index */
	CVT_V1724_FLASH_INDEX,							/*!< \brief Flash index */
	CVT_V1724_DUMMY16_INDEX,						/*!< \brief Dummy 16 register index */
	CVT_V1724_DUMMY32_INDEX,						/*!< \brief Dummy 32 register index */
	CVT_V1724_POST_TRIG_INDEX,						/*!< \brief Post trigger register index */
	CVT_V1724_FRONT_PANEL_IO_INDEX,					/*!< \brief Front panel IO register index */
	CVT_V1724_FRONT_PANEL_IO_CTRL_INDEX,			/*!< \brief Front panel IO Control register index */
//
// CH 0
	CVT_V1724_CH0_THRESHOLD_INDEX,					/*!< \brief CH 0 Threshold register relative index */
	CVT_V1724_CH0_THR_SAMPLE_INDEX,					/*!< \brief CH 0 Over/Under Threshold Samples register relative index */
	CVT_V1724_CH0_STATUS_INDEX,						/*!< \brief CH 0 status register relative index */
	CVT_V1724_CH0_FW_REV_INDEX,						/*!< \brief CH 0 firmware revision register relative index */
	CVT_V1724_CH0_READ_BLK_TRIG_INDEX,				/*!< \brief CH 0 Read Block Trigger register relative index */
	CVT_V1724_CH0_BLK_WRI_NUM_INDEX,				/*!< \brief CH 0 Number of Blocks Written register relative index */
	CVT_V1724_CH0_DAC_CONF_INDEX,					/*!< \brief CH 0 DAC Data Configuration register relative index */
	CVT_V1724_CH0_CONF_INDEX,						/*!< \brief CH 0 Configuration register relative index */
	CVT_V1724_CH0_DEBUG_INDEX,						/*!< \brief CH 0 Debug register relative index */
//
// CH 1
	CVT_V1724_CH1_THRESHOLD_INDEX,					/*!< \brief CH 1 Threshold register relative index */
	CVT_V1724_CH1_THR_SAMPLE_INDEX,					/*!< \brief CH 1 Over/Under Threshold Samples register relative index */
	CVT_V1724_CH1_STATUS_INDEX,						/*!< \brief CH 1 status register relative index */
	CVT_V1724_CH1_FW_REV_INDEX,						/*!< \brief CH 1 firmware revision register relative index */
	CVT_V1724_CH1_READ_BLK_TRIG_INDEX,				/*!< \brief CH 1 Read Block Trigger register relative index */
	CVT_V1724_CH1_BLK_WRI_NUM_INDEX,				/*!< \brief CH 1 Number of Blocks Written register relative index */
	CVT_V1724_CH1_DAC_CONF_INDEX,					/*!< \brief CH 1 DAC Data Configuration register relative index */
	CVT_V1724_CH1_CONF_INDEX,						/*!< \brief CH 1 Configuration register relative index */
	CVT_V1724_CH1_DEBUG_INDEX,						/*!< \brief CH 1 Debug register relative index */
//
// CH 2
	CVT_V1724_CH2_THRESHOLD_INDEX,					/*!< \brief CH 2 Threshold register relative index */
	CVT_V1724_CH2_THR_SAMPLE_INDEX,					/*!< \brief CH 2 Over/Under Threshold Samples register relative index */
	CVT_V1724_CH2_STATUS_INDEX,						/*!< \brief CH 2 status register relative index */
	CVT_V1724_CH2_FW_REV_INDEX,						/*!< \brief CH 2 firmware revision register relative index */
	CVT_V1724_CH2_READ_BLK_TRIG_INDEX,				/*!< \brief CH 2 Read Block Trigger register relative index */
	CVT_V1724_CH2_BLK_WRI_NUM_INDEX,				/*!< \brief CH 2 Number of Blocks Written register relative index */
	CVT_V1724_CH2_DAC_CONF_INDEX,					/*!< \brief CH 2 DAC Data Configuration register relative index */
	CVT_V1724_CH2_CONF_INDEX,						/*!< \brief CH 2 Configuration register relative index */
	CVT_V1724_CH2_DEBUG_INDEX,						/*!< \brief CH 2 Debug register relative index */
//
// CH 3
	CVT_V1724_CH3_THRESHOLD_INDEX,					/*!< \brief CH 3 Threshold register relative index */
	CVT_V1724_CH3_THR_SAMPLE_INDEX,					/*!< \brief CH 3 Over/Under Threshold Samples register relative index */
	CVT_V1724_CH3_STATUS_INDEX,						/*!< \brief CH 3 status register relative index */
	CVT_V1724_CH3_FW_REV_INDEX,						/*!< \brief CH 3 firmware revision register relative index */
	CVT_V1724_CH3_READ_BLK_TRIG_INDEX,				/*!< \brief CH 3 Read Block Trigger register relative index */
	CVT_V1724_CH3_BLK_WRI_NUM_INDEX,				/*!< \brief CH 3 Number of Blocks Written register relative index */
	CVT_V1724_CH3_DAC_CONF_INDEX,					/*!< \brief CH 3 DAC Data Configuration register relative index */
	CVT_V1724_CH3_CONF_INDEX,						/*!< \brief CH 3 Configuration register relative index */
	CVT_V1724_CH3_DEBUG_INDEX,						/*!< \brief CH 3 Debug register relative index */
//
// CH 4
	CVT_V1724_CH4_THRESHOLD_INDEX,					/*!< \brief CH 4 Threshold register relative index */
	CVT_V1724_CH4_THR_SAMPLE_INDEX,					/*!< \brief CH 4 Over/Under Threshold Samples register relative index */
	CVT_V1724_CH4_STATUS_INDEX,						/*!< \brief CH 4 status register relative index */
	CVT_V1724_CH4_FW_REV_INDEX,						/*!< \brief CH 4 firmware revision register relative index */
	CVT_V1724_CH4_READ_BLK_TRIG_INDEX,				/*!< \brief CH 4 Read Block Trigger register relative index */
	CVT_V1724_CH4_BLK_WRI_NUM_INDEX,				/*!< \brief CH 4 Number of Blocks Written register relative index */
	CVT_V1724_CH4_DAC_CONF_INDEX,					/*!< \brief CH 4 DAC Data Configuration register relative index */
	CVT_V1724_CH4_CONF_INDEX,						/*!< \brief CH 4 Configuration register relative index */
	CVT_V1724_CH4_DEBUG_INDEX,						/*!< \brief CH 4 Debug register relative index */
//
// CH 5
	CVT_V1724_CH5_THRESHOLD_INDEX,					/*!< \brief CH 5 Threshold register relative index */
	CVT_V1724_CH5_THR_SAMPLE_INDEX,					/*!< \brief CH 5 Over/Under Threshold Samples register relative index */
	CVT_V1724_CH5_STATUS_INDEX,						/*!< \brief CH 5 status register relative index */
	CVT_V1724_CH5_FW_REV_INDEX,						/*!< \brief CH 5 firmware revision register relative index */
	CVT_V1724_CH5_READ_BLK_TRIG_INDEX,				/*!< \brief CH 5 Read Block Trigger register relative index */
	CVT_V1724_CH5_BLK_WRI_NUM_INDEX,				/*!< \brief CH 5 Number of Blocks Written register relative index */
	CVT_V1724_CH5_DAC_CONF_INDEX,					/*!< \brief CH 5 DAC Data Configuration register relative index */
	CVT_V1724_CH5_CONF_INDEX,						/*!< \brief CH 5 Configuration register relative index */
	CVT_V1724_CH5_DEBUG_INDEX,						/*!< \brief CH 5 Debug register relative index */
//
// CH 6
	CVT_V1724_CH6_THRESHOLD_INDEX,					/*!< \brief CH 6 Threshold register relative index */
	CVT_V1724_CH6_THR_SAMPLE_INDEX,					/*!< \brief CH 6 Over/Under Threshold Samples register relative index */
	CVT_V1724_CH6_STATUS_INDEX,						/*!< \brief CH 6 status register relative index */
	CVT_V1724_CH6_FW_REV_INDEX,						/*!< \brief CH 6 firmware revision register relative index */
	CVT_V1724_CH6_READ_BLK_TRIG_INDEX,				/*!< \brief CH 6 Read Block Trigger register relative index */
	CVT_V1724_CH6_BLK_WRI_NUM_INDEX,				/*!< \brief CH 6 Number of Blocks Written register relative index */
	CVT_V1724_CH6_DAC_CONF_INDEX,					/*!< \brief CH 6 DAC Data Configuration register relative index */
	CVT_V1724_CH6_CONF_INDEX,						/*!< \brief CH 6 Configuration register relative index */
	CVT_V1724_CH6_DEBUG_INDEX,						/*!< \brief CH 6 Debug register relative index */
//
// CH 7
	CVT_V1724_CH7_THRESHOLD_INDEX,					/*!< \brief CH 7 Threshold register relative index */
	CVT_V1724_CH7_THR_SAMPLE_INDEX,					/*!< \brief CH 7 Over/Under Threshold Samples register relative index */
	CVT_V1724_CH7_STATUS_INDEX,						/*!< \brief CH 7 status register relative index */
	CVT_V1724_CH7_FW_REV_INDEX,						/*!< \brief CH 7 firmware revision register relative index */
	CVT_V1724_CH7_READ_BLK_TRIG_INDEX,				/*!< \brief CH 7 Read Block Trigger register relative index */
	CVT_V1724_CH7_BLK_WRI_NUM_INDEX,				/*!< \brief CH 7 Number of Blocks Written register relative index */
	CVT_V1724_CH7_DAC_CONF_INDEX,					/*!< \brief CH 7 DAC Data Configuration register relative index */
	CVT_V1724_CH7_CONF_INDEX,						/*!< \brief CH 7 Configuration register relative index */
	CVT_V1724_CH7_DEBUG_INDEX,						/*!< \brief CH 7 Debug register relative index */

} CVT_V792_REG_INDEX;
												
#define CVT_V1724_MAX_CHANNEL			8			/*!< \brief The number of channels */

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V1724_CONTROL_MSK
*   \brief   Control register bitmasks
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
	CVT_V1724_CTRL_SWITCH_CLKEN_MSK		= 0x0001,		/*!< Enable SYNC generation bit */
	CVT_V1724_CTRL_FREQ_SEL_MSK			= 0x0002,		/*!< SYNC frequency selection bit */
	CVT_V1724_CTRL_BERR_ENABLE_MSK		= 0x0010,		/*!< Bus error enable bit */
} CVT_V1724_CONTROL_MSK;

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V1724_STATUS_MSK
*   \brief   Status register bitmasks
*   \todo    To be defined
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
	CVT_V1724_STS_FREE_MSK				= 0x0001,		/*!< Dummy Free bit */
} CVT_V1724_STATUS_MSK;

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V1724_TRIGGER_ENABLE_MSK
*   \brief   Trigger enable bitmasks
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
	CVT_V1724_TRGEN_CH0_MSK				= 0x0001,		/*!< Enable CH 0 trigger bit */
	CVT_V1724_TRGEN_CH1_MSK				= 0x0002,		/*!< Enable CH 1 trigger bit */
	CVT_V1724_TRGEN_CH2_MSK				= 0x0008,		/*!< Enable CH 2 trigger bit */
	CVT_V1724_TRGEN_CH3_MSK				= 0x0010,		/*!< Enable CH 3 trigger bit */
	CVT_V1724_TRGEN_CH4_MSK				= 0x0040,		/*!< Enable CH 4 trigger bit */
	CVT_V1724_TRGEN_CH5_MSK				= 0x0080,		/*!< Enable CH 5 trigger bit */
	CVT_V1724_TRGEN_CH6_MSK				= 0x0200,		/*!< Enable CH 6 trigger bit */
	CVT_V1724_TRGEN_CH7_MSK				= 0x0400,		/*!< Enable CH 7 trigger bit */
	CVT_V1724_TRGEN_ENABLE_MSK			= 0x2000,		/*!< Global channel triggers enable bit */
	CVT_V1724_TRGEN_EXT_MSK				= 0x4000,		/*!< External trigger enable bit */
	CVT_V1724_TRGEN_SW_MSK				= 0x8000,		/*!< Software trigger enable bit */
} CVT_V1724_TRIGGER_ENABLE_MSK;

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V1724_CH_CONTROL_MSK
*   \brief   CH Control register bitmasks
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
	CVT_V1724_CHCTRL_SAMPLE_ACQ_MSK			= 0x00000001,	/*!< Enable Sample Acquisition ( 0 --> Window Acquisition) bit */
	CVT_V1724_CHCTRL_TRG_OVERLAP_MSK		= 0x00000002,	/*!< Trigger Overlapping enable bit */
	CVT_V1724_CHCTRL_MEM_ENABLE_MSK			= 0x00000004,	/*!< Memory chip enable Not stopped bit */
	CVT_V1724_CHCTRL_MEM_TEST_MSK			= 0x00000008,	/*!< Memory Test enable bit */
	CVT_V1724_CHCTRL_MEM_RANDOM_ACC_MSK		= 0x00000010,	/*!< Memory random access bit */
	CVT_V1724_CHCTRL_TRG_IN_EN_MSK			= 0x00000020,	/*!< Trigger input enable bit */
	CVT_V1724_CHCTRL_TRG_OUT_UNDER_EN_MSK	= 0x00000040,	/*!< Trigger Output on Input Under ( 0 --> Over) Threshold enable bit */
	CVT_V1724_CHCTRL_TRG_OUT_EN_MSK			= 0x00000080,	/*!< Trigger Output Generation enable bit */
	CVT_V1724_CHCTRL_BIST_TEST_MSK			= 0x00000100,	/*!< BIST Test and Continuos Write enable bit */
	CVT_V1724_CHCTRL_READ_MEM_MSK			= 0x00000200,	/*!< Read Memory Test enable bit */
} CVT_V1724_CH_CONTROL_MSK;

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V1724_CH_READ_CONF_MSK
*   \brief   CH Read configuration register bitmasks
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
	CVT_V1724_CHRDC_BLOCK_ADD_MSK		= 0x00000FFF,	/*!< Address of Block to be read bits */
	CVT_V1724_CHRDC_SAMPLE_NUM_MSK		= 0x003FF000,	/*!< Number of samples to be read bits */
	CVT_V1724_CHRDC_OFFSET_MSK			= 0xFFC00000,	/*!< Offset of Address Data to be read bits */
} CVT_V1724_CH_READ_CONF_MSK;

#define CVT_V1724_GET_CH_READ_CONF_BLOCK_ADD( reg)			(((UINT32)reg)& CVT_V1724_CHRDC_BLOCK_ADD_MSK)																		/*!< \brief Extract the Address of Block to be read from UINT32 value */
#define CVT_V1724_SET_CH_READ_CONF_BLOCK_ADD( reg, value)	reg= (((UINT32)reg)& ~CVT_V1724_CHRDC_BLOCK_ADD_MSK)| ((UINT32)value& CVT_V1724_CHRDC_BLOCK_ADD_MSK)				/*!< \brief Sets the Address of Block to be read into UINT32 value */

#define CVT_V1724_GET_CH_READ_CONF_SAMPLE_NUM( reg)			((((UINT32)reg)& CVT_V1724_CHRDC_SAMPLE_NUM_MSK)>> 12)																/*!< \brief Extract the Number of samples to be read from UINT32 value */
#define CVT_V1724_SET_CH_READ_CONF_SAMPLE_NUM( reg, value)	reg= (((UINT32)reg)& ~CVT_V1724_CHRDC_SAMPLE_NUM_MSK)| ((((UINT32)value)<< 12)& CVT_V1724_CHRDC_SAMPLE_NUM_MSK)		/*!< \brief Sets the Number of samples to be read into UINT32 value */

#define CVT_V1724_GET_CH_READ_CONF_OFFSET( reg)				((((UINT32)reg)& CVT_V1724_CHRDC_OFFSET_MSK)>> 22)																	/*!< \brief Extract the Offset of Address Data  from UINT32 value */
#define CVT_V1724_SET_CH_READ_CONF_OFFSET( reg, value)		reg= (((UINT32)reg)& ~CVT_V1724_CHRDC_OFFSET_MSK)| ((((UINT32)value)<< 22)& CVT_V1724_CHRDC_OFFSET_MSK)				/*!< \brief Sets the Offset of Address Data into UINT32 value */

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V1724_CH_CONF_MSK
*   \brief   CH Configuration register bitmasks
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
	CVT_V1724_CHCONF_DITHER_MSK			= 0x00000001,	/*!< CH Dither bit */
	CVT_V1724_CHCONF_OUTPUT_MODE_MSK	= 0x00000002,	/*!< CH Mode of Output Binary Code bit */
	CVT_V1724_CHCONF_RND_MSK			= 0x00000004,	/*!< CH to Randomize the Output Binary Code bit */
} CVT_V1724_CH_CONF_MSK;

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V1724_CH_CONF_MSK
*   \brief   CH Configuration register bitmasks
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
	CVT_V1724_FPIO_CTRL_TTL_MSK			= 0x00000001,	/*!< TTL\NIM external signal  */
} CVT_V1724_FRONT_PANEL_IO_CTRL_MSK;

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V1724_DAC_CONF_MSK
*   \brief   CH DAC data configuration register bitmasks
*   \todo    Bits to define
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
	CVT_V1724_CHDAC_DATA_MSK			= 0x0000FFFF,	/*!< DAC data bits */
	CVT_V1724_CHDAC_SET_A_MSK			= 0x00100000,	/*!< Set data for DAC A */
	CVT_V1724_CHDAC_SET_B_MSK			= 0x00240000,	/*!< Set data for DAC B */
} CVT_V1724_DAC_CONF_MSK;


#define CVT_V1724_GET_CH_DAC_CONF( reg)			(((UINT32)reg)& CVT_V1724_CHDAC_DATA_MSK)																/*!< \brief Extract DAC's data from UINT32 value */
#define CVT_V1724_SET_CH_DAC_CONF( reg, value)	reg= (((UINT32)reg)& ~CVT_V1724_CHDAC_DATA_MSK)| ((UINT32)value& CVT_V1724_CHDAC_DATA_MSK)				/*!< \brief Sets DAC's data into UINT32 value */
  
////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V1724_CH_STATUS_MSK
*   \brief   CH status register bitmasks
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
	CVT_V1724_CHSTS_FIFO_FULL_MSK		= 0x00000001,	/*!< CH Descriptor FIFO full Status bit */
	CVT_V1724_CHSTS_FIFO_EMPTY_MSK		= 0x00000002,	/*!< CH Descriptor FIFO empty Status bit */
	CVT_V1724_CHSTS_DAC_BUSY_MSK		= 0x00000004,	/*!< CH DAC Busy Status bit */
	CVT_V1724_CHSTS_BIST_END_MSK		= 0x00000008,	/*!< CH BIST end Status bit */
	CVT_V1724_CHSTS_BIST_OK_MSK			= 0x00000010,	/*!< CH BIST result bit */
	CVT_V1724_CHSTS_BLOCK_REM_OK_MSK	= 0x00000020,	/*!< CH Block Remove result bit */
} CVT_V1724_CH_STATUS_MSK;
  
////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V1724_CH_BLKSIZE
*   \brief   The channel Samples' Number for each block
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
	CVT_V1724_CHBKSZ_512K= 0,			/*!< 2 blocks,    262144 32-bit memory locations (1024K byte/block), 512K samples/block */
	CVT_V1724_CHBKSZ_256K,				/*!< 4 blocks,    131072 32-bit memory locations ( 512K byte/block), 256K samples/block */
	CVT_V1724_CHBKSZ_128K,				/*!< 8 blocks,     65536 32-bit memory locations ( 256K byte/block), 128K samples/block */
	CVT_V1724_CHBKSZ_64K,				/*!< 16 blocks,    32768 32-bit memory locations ( 128K byte/block),  64K samples/block */
	CVT_V1724_CHBKSZ_32K,				/*!< 32 blocks,    16384 32-bit memory locations (  64K byte/block),  32K samples/block */
	CVT_V1724_CHBKSZ_16K,				/*!< 64 blocks,     8192 32-bit memory locations (  32K byte/block),  16K samples/block */
	CVT_V1724_CHBKSZ_8K,				/*!< 128 blocks,    4096 32-bit memory locations (  16K byte/block),   8K samples/block */
	CVT_V1724_CHBKSZ_4K,				/*!< 256 blocks,    2048 32-bit memory locations (   8K byte/block),   4K samples/block */
	CVT_V1724_CHBKSZ_2K,				/*!< 512 blocks,    1024 32-bit memory locations (   4K byte/block),   2K samples/block */
	CVT_V1724_CHBKSZ_1K,				/*!< 1024 blocks,    512 32-bit memory locations (   2K byte/block),   1K samples/block */
	CVT_V1724_CHBKSZ_512,				/*!< 2048 blocks,    256 32-bit memory locations (   1K byte/block),  512 samples/block */
} CVT_V1724_CH_BLKSIZE;

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V1724_MCST_CBLT_CTRL_MSK
*   \brief   V1724 CVT_V1724/MCST Control register bit masks
*            
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
	CVT_V1724_MCCTRL_DISABLED_BOARD_MSK			= 0x0000,		/*!< Disabled Board mask */
	CVT_V1724_MCCTRL_LAST_BOARD_MSK				= 0x0001,		/*!< Last Board mask */
	CVT_V1724_MCCTRL_FIRST_BOARD_MSK			= 0x0002,		/*!< First Board mask */
	CVT_V1724_MCCTRL_MID_BOARD_MSK				= 0x0003,		/*!< Middle Board mask */
} CVT_V1724_MCST_CBLT_CTRL_MSK;

////////////////////////////////////////////////////////////////////////////////////////////////
// Global variables declaration
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
// Global methods declaration
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
//
//     B O A R D S   H A N D L I N G
//
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_open( cvt_V1724_data* p_data, UINT16 base_address, long vme_handle);
*   \brief   V1724 VME boards data initialization
*            
*            Provides specific handling for V1724 boards opening.
*   \param   p_data Pointer to board data
*   \param   base_address The board base address (MSW)
*   \param   vme_handle The VME handle
*   \return  TRUE: board successfully opened
*   \note    Must be called before any other board specific API.
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_open( cvt_V1724_data* p_data, UINT16 base_address, long vme_handle);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_close( cvt_V1724_data* p_data);
*   \brief   V1724 VME boards closing and resource free
*            
*            Provides specific handling for V1724 boards closing.
*   \param   p_data Pointer to board data
*   \return  TRUE: board successfully closed
*   \note    Must be called when done with any other board specific API.
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_close( cvt_V1724_data* p_data);

////////////////////////////////////////////////////////////////////////////////////////////////
//
//     L E V E L   0   A P I s
//
////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////
//
//     L E V E L   1   A P I s
//
////////////////////////////////////////////////////////////////////////////////////////////////

	
////////////////////////////////////////////////////////////////////////////////////////////////
//
//     L E V E L   2   A P I s
//
////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_read_buffer( cvt_V1724_data* p_data, UINT8 ch_index, UINT16* p_buff, UINT32* p_buff_size);
*   \brief   Reads data from Multiple event buffer and stores to user buffer
*            
*            Call cvt_FIFO_BLT_read for Multiple Event Buffer and stores data into user buffer.
*            If the returned data is just a V1724 not valid datum, this is discarded.
*   \param   p_data Pointer to board data
*   \param   ch_index The channel index 
*   \param   p_buff The target buffer: caller allocated
*   \param   p_buff_size The buffer size (16 bit word). On exit will holds the number of words really read.
*   \return  TRUE: Procedure successfully executed
*   \sa      cvt_FIFO_BLT_read
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_read_buffer( cvt_V1724_data* p_data, UINT8 ch_index, UINT16* p_buff, UINT32* p_buff_size);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_set_trigger_mode( cvt_V1724_data* p_data, BOOL falling_edge_enable, BOOL trigger_in_enable, BOOL trigger_out_enable, BOOL ext_trigger_enable, BOOL sw_trigger_enable, UINT8 ch_trigger_enable_msk, BOOL trigger_overlap_enable, UINT32 post_trigger);
*   \brief   Setups the triggering mode parameters.
*            
*            Setups the relevant parameters for the triggering mode.
*   \param   p_data Pointer to board data
*   \param   falling_edge_enable Enable/Disable triggering on falling edge
*   \param   trigger_in_enable Enable/Disable channels' trigger input
*   \param   trigger_out_enable Enable/Disable channels' trigger output
*   \param   ext_trigger_enable Enable/Disable the external trigger.
*   \param   sw_trigger_enable Enable/Disable the software trigger.
*   \param   ch_trigger_enable_msk Enable/Disable trigger from channels thresholds comparison. If bit i of the mask is setted, the channel i feature is setted.
*   \param   trigger_overlap_enable Enable/Disable the trigger overlap feature.
*   \param   post_trigger The amount of post trigger samples
*   \return  TRUE: Procedure successfully executed
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_set_trigger_mode( cvt_V1724_data* p_data, BOOL falling_edge_enable, BOOL trigger_in_enable, BOOL trigger_out_enable, BOOL ext_trigger_enable, BOOL sw_trigger_enable, UINT8 ch_trigger_enable_msk, BOOL trigger_overlap_enable, UINT32 post_trigger);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_set_acquisition_mode( cvt_V1724_data* p_data, BOOL sample_enable, CVT_V1724_CH_BLKSIZE block_size);
*   \brief   Setups the acquisition mode parameters.
*            
*            Setups the relevant parameters for the acquisition mode.
*   \param   p_data Pointer to board data
*   \param   sample_enable Enable/Disable the sample acquisition mode: if disabled the acquisition mode is windowed
*   \param   block_size Setup the samples' number per block size foreach channel: it must be a valid \ref CVT_V1724_CH_BLKSIZE identifier.
*   \return  TRUE: Procedure successfully executed
*   \sa      CVT_V1724_CH_BLKSIZE
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_set_acquisition_mode( cvt_V1724_data* p_data, BOOL sample_enable, CVT_V1724_CH_BLKSIZE block_size);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_set_dither_enable( cvt_V1724_data* p_data, UINT8 ch_msk, BOOL dither_value);
*   \brief   Setups the channels' dithering feature.
*            
*            Enables/Disables the dither feature for each channel specified with ch_msk.
*   \param   p_data Pointer to board data
*   \param   ch_msk The channel mask 
*   \param   dither_value Set to TRUE to enable the dither channels in the mask, FALSE to disable
*   \return  TRUE: Procedure successfully executed
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_set_dither_enable( cvt_V1724_data* p_data, UINT8 ch_msk, BOOL dither_value);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_set_interrupt( cvt_V1724_data* p_data, UINT8 level, UINT8 vector, UINT8 event_number);
*   \brief   Setups interrupt parameters.
*            
*            Setups the relevant parameters for interrupt usage.
*   \param   p_data Pointer to board data
*   \param   level The interrupt level.
*   \param   vector The interrupt vector.
*   \param   event_number The number of events to get an interrupt (setting 0 interrupt feature is disabled).
*   \return  TRUE: Procedure successfully executed
*   \todo    event_number not jet implemented on fpga board.
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_set_interrupt( cvt_V1724_data* p_data, UINT8 level, UINT8 vector, UINT8 event_number);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_set_readout_mode( cvt_V1724_data* p_data, BOOL enable_bus_error, UINT16 BLT_event_number);
*   \brief   Setups data readout mode parameters.
*            
*            Setups the relevant parameters for data readout.
*   \param   p_data Pointer to board data
*   \param   enable_bus_error Enable bus error: the module is enabled to generate a Bus error to finish a block transfer. 
*   \param   BLT_event_number The number of events to readout foreach BLT cycle.
*   \return  TRUE: Procedure successfully executed
*   \todo    To be defined
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_set_readout_mode( cvt_V1724_data* p_data, BOOL enable_bus_error, UINT16 BLT_event_number);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_software_reset( cvt_V1724_data* p_data);
*   \brief   Performs a software reset.
*            
*            Writes a dummy value into SW_RESET_REGISTER register.
*   \param   p_data Pointer to board data
*   \return  TRUE: Procedure successfully executed
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_software_reset( cvt_V1724_data* p_data);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_data_clear( cvt_V1724_data* p_data);
*   \brief   Performs a data clear.
*            
*            Writes a dummy value into SW_CLEAR_REGISTER register.
*   \param   p_data Pointer to board data
*   \return  TRUE: Procedure successfully executed
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_data_clear( cvt_V1724_data* p_data);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_set_channel_offset( cvt_V1724_data* p_data, UINT8 ch_msk, UINT16 offset_value);
*   \brief   Sets channels' offset.
*            
*            Sets the offset value for each channel specified with ch_msk.
*   \param   p_data Pointer to board data
*   \param   ch_msk The channel mask 
*   \param   offset_value The offset value to be applied to specified channels.
*   \return  TRUE: Procedure successfully executed
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_set_channel_offset( cvt_V1724_data* p_data, UINT8 ch_msk, UINT16 offset_value);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_set_channel_trigger( cvt_V1724_data* p_data, UINT8 ch_msk, UINT32 trigger_threshold, UINT32 threshold_samples);
*   \brief   Sets channels' trigger parameters.
*            
*            Sets the relevant trigger parameters for each channel specified with ch_msk.
*   \param   p_data Pointer to board data
*   \param   ch_msk The channel mask 
*   \param   trigger_threshold The value of the trigger threshold
*   \param   threshold_samples The hysteresis on threshold comparison 
*   \return  TRUE: Procedure successfully executed
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_set_channel_trigger( cvt_V1724_data* p_data, UINT8 ch_msk, UINT32 trigger_threshold, UINT32 threshold_samples);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_set_front_panel_IO( cvt_V1724_data* p_data, BOOL use_TTL);
*   \brief   Sets front panel's IO.
*            
*            Sets the relevant parameters for the front panel's IO.
*   \param   p_data Pointer to board data
*   \param   use_TTL If set, the external signals standard is TTL type, elsewhere NIM type is assumed
*   \return  TRUE: Procedure successfully executed
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_set_front_panel_IO( cvt_V1724_data* p_data, BOOL use_TTL);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_software_trigger( cvt_V1724_data* p_data);
*   \brief   Performs a software trigger.
*            
*            Sends a software trigger. Software triggers must be enabled
*   \param   p_data Pointer to board data
*   \return  TRUE: Procedure successfully executed
*   \sa      cvt_V1724_set_trigger_mode
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_software_trigger( cvt_V1724_data* p_data);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_get_channel_status( cvt_V1724_data* p_data, UINT8 ch_index, BOOL *p_is_dac_busy, BOOL *p_is_fifo_full, BOOL *p_is_fifo_empty, BOOL *p_is_block_remove_ok);
*   \brief   Gets information about board status.
*            
*   \param   p_data Pointer to board data.
*   \param   ch_index The index of the channel to test.
*   \param   p_is_dac_busy Returns the channel's DAC_BUSY flag status. 
*   \param   p_is_fifo_full Returns the channel's FIFO_FULL flag status. 
*   \param   p_is_fifo_empty Returns the channel's FIFO_EMPTY flag status. 
*   \param   p_is_block_remove_ok Returns the channel's BLOCK_REM_OK flag status. 
*   \return  TRUE: Procedure successfully executed.
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_get_channel_status( cvt_V1724_data* p_data, UINT8 ch_index, BOOL *p_is_dac_busy, BOOL *p_is_fifo_full, BOOL *p_is_fifo_almost_full, BOOL *p_is_block_remove_ok);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_get_system_info( cvt_V1724_data* p_data, UINT16 *p_firmware_rev, UINT16 *p_serial_number);
*   \brief   Gets board's system information.
*            
*            Reads the firmware revision register and the serial number.
*   \param   p_data Pointer to board data
*   \param   p_firmware_rev The firmare release (MSB major release, LSB minor release).
*   \param   p_serial_number The serial number.
*   \return  TRUE: Procedure successfully executed
*   \todo    p_serial_number not jet implemented into board fpga 
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_get_system_info( cvt_V1724_data* p_data, UINT16 *p_firmware_rev, UINT16 *p_serial_number);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1724_set_MCST_CBLT( cvt_V1724_data* p_data, UINT8 address, MCST_CBLT_board_pos pos);
*   \brief   Setups MCST/CBLT parameters this board.
*            
*            Setups the relevant parameters for MCST/CBLT usage.
*   \param   p_data Pointer to board data.
*   \param   address The MCST/CBLT address.
*   \param   pos The board position into the MCST / CBLT chain: it must be a \ref MCST_CBLT_board_pos identifier
*   \return  TRUE: Procedure successfully executed
*   \sa      MCST_CBLT_board_pos
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1724_set_MCST_CBLT( cvt_V1724_data* p_data, UINT8 address, MCST_CBLT_board_pos pos);

#endif  // __CVT_V1724_DEF_H
