////////////////////////////////////////////////////////////////////////////////////////////////
/*! \file    cvt_V812.c
*   \brief   V812 VME board implementation
*   \author  NDA
*   \version 1.0
*   \date    03/2006
*            
*            Provides methods, properties and defines to handle with V812 VME boards
*/
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
// File includes
////////////////////////////////////////////////////////////////////////////////////////////////
#include <memory.h>
#include <string.h>
#include "cvt_V812.h"

////////////////////////////////////////////////////////////////////////////////////////////////
// File local defines
////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////
// Static variables declaration
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \var     static const cvt_reg_table CVT_V812_REG_TABLE[]
*   \brief   The board registers table
*            
*            Provides an entry for each declared register: keep synched with \ref CVT_V812_REG_INDEX
*/
////////////////////////////////////////////////////////////////////////////////////////////////
static const cvt_reg_table CVT_V812_REG_TABLE[]=
{
	{ CVT_V812_THRESHOLD_0_ADD,		CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 0 register */
	{ CVT_V812_THRESHOLD_1_ADD,		CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 1 register */
	{ CVT_V812_THRESHOLD_2_ADD,		CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 2 register */
	{ CVT_V812_THRESHOLD_3_ADD,		CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 3 register */
	{ CVT_V812_THRESHOLD_4_ADD,		CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 4 register */
	{ CVT_V812_THRESHOLD_5_ADD,		CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 5 register */
	{ CVT_V812_THRESHOLD_6_ADD,		CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 6 register */
	{ CVT_V812_THRESHOLD_7_ADD,		CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 7 register */
	{ CVT_V812_THRESHOLD_8_ADD,		CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 8 register */
	{ CVT_V812_THRESHOLD_9_ADD,		CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 9 register */
	{ CVT_V812_THRESHOLD_10_ADD,	CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 10 register */
	{ CVT_V812_THRESHOLD_11_ADD,	CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 11 register */
	{ CVT_V812_THRESHOLD_12_ADD,	CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 12 register */
	{ CVT_V812_THRESHOLD_13_ADD,	CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 13 register */
	{ CVT_V812_THRESHOLD_14_ADD,	CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 14 register */
	{ CVT_V812_THRESHOLD_15_ADD,	CVT_V812_THRESHOLD_AM,		CVT_V812_THRESHOLD_DATA_SIZE},				/*!< \brief Threshold 15 register */
	{ CVT_V812_OUT_WIDTH_0_7_ADD,	CVT_V812_OUT_WIDTH_0_7_AM,	CVT_V812_OUT_WIDTH_0_7_DATA_SIZE},			/*!< \brief Output width register Ch 0-7 */
	{ CVT_V812_OUT_WIDTH_8_15_ADD,	CVT_V812_OUT_WIDTH_8_15_AM,	CVT_V812_OUT_WIDTH_8_15_DATA_SIZE},			/*!< \brief Output width register Ch 8-15 */
	{ CVT_V812_DEAD_TIME_0_7_ADD,	CVT_V812_DEAD_TIME_0_7_AM,	CVT_V812_DEAD_TIME_0_7_DATA_SIZE},			/*!< \brief Dead time register Ch 0-7 */
	{ CVT_V812_DEAD_TIME_8_15_ADD,	CVT_V812_DEAD_TIME_8_15_AM,	CVT_V812_DEAD_TIME_8_15_DATA_SIZE},			/*!< \brief Dead time register Ch 8-15 */
	{ CVT_V812_MAJORITY_ADD,		CVT_V812_MAJORITY_AM,		CVT_V812_MAJORITY_DATA_SIZE},				/*!< \brief CVT_V812_MAJORITY threshold register */
	{ CVT_V812_PATTERN_INHIBIT_ADD,	CVT_V812_PATTERN_INHIBIT_AM,CVT_V812_PATTERN_INHIBIT_DATA_SIZE},		/*!< \brief Pattern inhibit register */
	{ CVT_V812_TEST_PULSE_ADD,		CVT_V812_TEST_PULSE_AM,		CVT_V812_TEST_PULSE_DATA_SIZE},				/*!< \brief Test pulse register relative */
	{ CVT_V812_FIXED_CODE_ADD,		CVT_V812_FIXED_CODE_AM,		CVT_V812_FIXED_CODE_DATA_SIZE},				/*!< \brief Fixed code register relative */
	{ CVT_V812_MANUFACTURER_ADD,	CVT_V812_MANUFACTURER_AM,	CVT_V812_MANUFACTURER_DATA_SIZE},			/*!< \brief Manufacturer and Module type register relative */
	{ CVT_V812_VERSION_ADD,			CVT_V812_VERSION_AM,		CVT_V812_VERSION_DATA_SIZE},				/*!< \brief Version and serial number register relative */
};
////////////////////////////////////////////////////////////////////////////////////////////////
// Static methods declaration
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
// Global visible variables declaration
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
//
//     B O A R D S   H A N D L I N G
//
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_open( cvt_V812_data* p_data, UINT16 base_address, long vme_handle)
{
	memset( p_data, 0, sizeof( cvt_V812_data));
	// basic data initialization
	if( !cvt_board_open( &p_data->m_common_data, base_address, vme_handle, CVT_V812_REG_TABLE))
		return FALSE;
	// board specific data initialization

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_close( cvt_V812_data* p_data)
{
	if( !cvt_board_close( &p_data->m_common_data))
		return FALSE;
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
//
//     L E V E L   0   A P I s
//
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
//
//     L E V E L   1   A P I s
//
////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_set_threshold( cvt_V812_data* p_data, cvt_V812_threshold_id threshold_id, UINT16 value)
{
	UINT16 reg_value= value;
	// Parameter checking
	switch( threshold_id)
	{
	case CVT_V812_THRESHOLD_ALL:						// Catch all the thresholds
		{
			// write every threshold
			int i;
			for( i= CVT_V812_THRESHOLD_0; i< CVT_V812_MAX_THRESHOLD; i++)
			{
				if( !cvt_write_reg( &p_data->m_common_data, CVT_V812_THRESHOLD_0_INDEX+ (i- CVT_V812_THRESHOLD_0), &reg_value))
				{
					TRACE1( "V812 THRESHOLD %i write failed !\n", i- CVT_V812_THRESHOLD_0);
					return FALSE;
				}
			}
		}
		break;
	case CVT_V812_THRESHOLD_0:							// Threshold 0 register
	case CVT_V812_THRESHOLD_1:							// Threshold 1 register
	case CVT_V812_THRESHOLD_2:							// Threshold 2 register
	case CVT_V812_THRESHOLD_3:							// Threshold 3 register
	case CVT_V812_THRESHOLD_4:							// Threshold 4 register
	case CVT_V812_THRESHOLD_5:							// Threshold 5 register
	case CVT_V812_THRESHOLD_6:							// Threshold 6 register
	case CVT_V812_THRESHOLD_7:							// Threshold 7 register
	case CVT_V812_THRESHOLD_8:							// Threshold 8 register
	case CVT_V812_THRESHOLD_9:							// Threshold 9 register
	case CVT_V812_THRESHOLD_10:							// Threshold 10 register
	case CVT_V812_THRESHOLD_11:							// Threshold 11 register
	case CVT_V812_THRESHOLD_12:							// Threshold 12 register
	case CVT_V812_THRESHOLD_13:							// Threshold 13 register
	case CVT_V812_THRESHOLD_14:							// Threshold 14 register
	case CVT_V812_THRESHOLD_15:							// Threshold 15 register
		// write specific threshold
		if( !cvt_write_reg( &p_data->m_common_data, CVT_V812_THRESHOLD_0_INDEX+ ( threshold_id- CVT_V812_THRESHOLD_0), &reg_value))
		{
			TRACE1( "V812 THRESHOLD %i write failed !\n", threshold_id- CVT_V812_THRESHOLD_0);
			return FALSE;
		}
	default:
		TRACE1( "V812 Unhandled threshold id %i !\n", threshold_id);
		return FALSE;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_set_output_width( cvt_V812_data* p_data, UINT8 value)
{
	UINT16 reg_value= value;
	// write registers
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V812_OUT_WIDTH_0_7_INDEX, &reg_value))
	{
		TRACE( "V812 OUT_WIDTH_0_7 write failed !\n");
		return FALSE;
	}
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V812_OUT_WIDTH_8_15_INDEX, &reg_value))
	{
		TRACE( "V812 OUT_WIDTH_8_15 write failed !\n");
		return FALSE;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_set_dead_time( cvt_V812_data* p_data, UINT8 value)
{
	UINT16 reg_value= value;
	// write registers
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V812_DEAD_TIME_0_7_INDEX, &reg_value))
	{
		TRACE( "V812 CVT_V812_DEAD_TIME_0_7 write failed !\n");
		return FALSE;
	}
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V812_DEAD_TIME_8_15_INDEX, &reg_value))
	{
		TRACE( "V812 CVT_V812_DEAD_TIME_8_15 write failed !\n");
		return FALSE;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_set_majority_threshold( cvt_V812_data* p_data, UINT8 value)
{
	UINT16 reg_value= value;
	// write register
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V812_MAJORITY_INDEX, &reg_value))
	{
		TRACE( "V812 CVT_V812_MAJORITY write failed !\n");
		return FALSE;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_set_pattern_inhibit( cvt_V812_data* p_data, UINT16 value)
{
	UINT16 reg_value= value;
	// write register
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V812_PATTERN_INHIBIT_INDEX, &reg_value))
	{
		TRACE( "V812 CVT_V812_PATTERN_INHIBIT write failed !\n");
		return FALSE;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_set_test_pulse( cvt_V812_data* p_data)
{
	UINT16 reg_value= 0;
	// write register
	if( !cvt_write_reg( &p_data->m_common_data, CVT_V812_TEST_PULSE_INDEX, &reg_value))
	{
		TRACE( "V812 CVT_V812_TEST_PULSE write failed !\n");
		return FALSE;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_get_fixed_code( cvt_V812_data* p_data, UINT16 *p_value)
{
	UINT16 reg_value= 0;
	// read register
	if( !cvt_read_reg( &p_data->m_common_data, CVT_V812_FIXED_CODE_INDEX, &reg_value))
	{
		TRACE( "V812 CVT_V812_FIXED_CODE read failed !\n");
		return FALSE;
	}
	*p_value= reg_value;
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_get_manufacturer_number( cvt_V812_data* p_data, UINT8 *p_value)
{
	UINT16 reg_value= 0;
	// read register
	if( !cvt_read_reg( &p_data->m_common_data, CVT_V812_MANUFACTURER_INDEX, &reg_value))
	{
		TRACE( "V812 CVT_V812_MANUFACTURER_NUMBER read failed !\n");
		return FALSE;
	}
	*p_value= (UINT8)( ((UINT16)(reg_value>> 10))& 0x3f);
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_get_module_type( cvt_V812_data* p_data, UINT16 *p_value)
{
	UINT16 reg_value= 0;
	// read register
	if( !cvt_read_reg( &p_data->m_common_data, CVT_V812_MANUFACTURER_INDEX, &reg_value))
	{
		TRACE( "V812 CVT_V812_MODULE_TYPE read failed !\n");
		return FALSE;
	}
	*p_value= (UINT16)( reg_value& 0x03ff);
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_get_version( cvt_V812_data* p_data, UINT8 *p_value)
{
	UINT16 reg_value= 0;
	// read register
	if( !cvt_read_reg( &p_data->m_common_data, CVT_V812_VERSION_INDEX, &reg_value))
	{
		TRACE( "V812 CVT_V812_VERSION read failed !\n");
		return FALSE;
	}
	*p_value= (UINT8)( ((UINT16)(reg_value>> 12))& 0x0f);
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V812_get_serial_number( cvt_V812_data* p_data, UINT16 *p_value)
{
	UINT16 reg_value= 0;
	// read register
	if( !cvt_read_reg( &p_data->m_common_data, CVT_V812_VERSION_INDEX, &reg_value))
	{
		TRACE( "V812 CVT_V812_SERIAL_NUMBER read failed !\n");
		return FALSE;
	}
	*p_value= (UINT16)( reg_value& 0x0fff);
	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////////////////////
//
//     L E V E L   2   A P I s
//
////////////////////////////////////////////////////////////////////////////////////////////////

